# pylint: disable=no-self-use,invalid-name


from __future__ import with_statement
from __future__ import absolute_import
import pytest

from allennlp.common.testing import AllenNlpTestCase
from allennlp.nn.decoding import GrammarState

def is_nonterminal(symbol     )        :
    if symbol == u'identity':
        return False
    if u'lambda ' in symbol:
        return False
    if symbol in set([u'x', u'y', u'z']):
        return False
    return True


class TestGrammarState(AllenNlpTestCase):
    def test_is_finished_just_uses_nonterminal_stack(self):
        state = GrammarState([u's'], {}, {}, {}, is_nonterminal)
        assert not state.is_finished()
        state = GrammarState([], {}, {}, {}, is_nonterminal)
        assert state.is_finished()

    def test_get_valid_actions_uses_top_of_stack(self):
        state = GrammarState([u's'], {}, {u's': [1, 2], u't': [3, 4]}, {}, is_nonterminal)
        assert state.get_valid_actions() == [1, 2]
        state = GrammarState([u't'], {}, {u's': [1, 2], u't': [3, 4]}, {}, is_nonterminal)
        assert state.get_valid_actions() == [3, 4]
        state = GrammarState([u'e'], {}, {u's': [1, 2], u't': [3, 4], u'e': []}, {}, is_nonterminal)
        assert state.get_valid_actions() == []

    def test_get_valid_actions_adds_lambda_productions(self):
        state = GrammarState([u's'], {(u's', u'x'): [u's']}, {u's': [1, 2]}, {u's -> x': 5}, is_nonterminal)
        assert state.get_valid_actions() == [1, 2, 5]
        # We're doing this assert twice to make sure we haven't accidentally modified the state.
        assert state.get_valid_actions() == [1, 2, 5]

    def test_get_valid_actions_adds_lambda_productions_only_for_correct_type(self):
        state = GrammarState([u't'],
                             {(u's', u'x'): [u't']},
                             {u's': [1, 2], u't': [3, 4]},
                             {u's -> x': 5},
                             is_nonterminal)
        assert state.get_valid_actions() == [3, 4]
        # We're doing this assert twice to make sure we haven't accidentally modified the state.
        assert state.get_valid_actions() == [3, 4]

    def test_take_action_gives_correct_next_states_with_non_lambda_productions(self):
        # state.take_action() doesn't read or change these objects, it just passes them through, so
        # we'll use some sentinels to be sure of that.
        valid_actions = object()
        action_indices = object()

        state = GrammarState([u's'], {}, valid_actions, action_indices, is_nonterminal)
        next_state = state.take_action(u's -> [t, r]')
        expected_next_state = GrammarState([u'r', u't'], {}, valid_actions, action_indices, is_nonterminal)
        assert next_state.__dict__ == expected_next_state.__dict__

        state = GrammarState([u'r', u't'], {}, valid_actions, action_indices, is_nonterminal)
        next_state = state.take_action(u't -> identity')
        expected_next_state = GrammarState([u'r'], {}, valid_actions, action_indices, is_nonterminal)
        assert next_state.__dict__ == expected_next_state.__dict__

    def test_take_action_crashes_with_mismatched_types(self):
        with pytest.raises(AssertionError):
            state = GrammarState([u's'], {}, {}, {}, is_nonterminal)
            state.take_action(u't -> identity')

    def test_take_action_gives_correct_next_states_with_lambda_productions(self):
        # state.take_action() doesn't read or change these objects, it just passes them through, so
        # we'll use some sentinels to be sure of that.
        valid_actions = object()
        action_indices = object()

        state = GrammarState([u't', u'<s,d>'], {}, valid_actions, action_indices, is_nonterminal)
        next_state = state.take_action(u'<s,d> -> [lambda x, d]')
        expected_next_state = GrammarState([u't', u'd'],
                                           {(u's', u'x'): [u'd']},
                                           valid_actions,
                                           action_indices,
                                           is_nonterminal)
        assert next_state.__dict__ == expected_next_state.__dict__

        state = expected_next_state
        next_state = state.take_action(u'd -> [<s,r>, d]')
        expected_next_state = GrammarState([u't', u'd', u'<s,r>'],
                                           {(u's', u'x'): [u'd', u'<s,r>']},
                                           valid_actions,
                                           action_indices,
                                           is_nonterminal)
        assert next_state.__dict__ == expected_next_state.__dict__

        state = expected_next_state
        next_state = state.take_action(u'<s,r> -> [lambda y, r]')
        expected_next_state = GrammarState([u't', u'd', u'r'],
                                           {(u's', u'x'): [u'd', u'r'], (u's', u'y'): [u'r']},
                                           valid_actions,
                                           action_indices,
                                           is_nonterminal)
        assert next_state.__dict__ == expected_next_state.__dict__

        state = expected_next_state
        next_state = state.take_action(u'r -> identity')
        expected_next_state = GrammarState([u't', u'd'],
                                           {(u's', u'x'): [u'd']},
                                           valid_actions,
                                           action_indices,
                                           is_nonterminal)
        assert next_state.__dict__ == expected_next_state.__dict__

        state = expected_next_state
        next_state = state.take_action(u'd -> x')
        expected_next_state = GrammarState([u't'],
                                           {},
                                           valid_actions,
                                           action_indices,
                                           is_nonterminal)
        assert next_state.__dict__ == expected_next_state.__dict__



u"""
A ``SqlTableContext`` represents the SQL context in which an utterance appears, with the grammar and
the valid actions.
"""

from __future__ import absolute_import
import re
from collections import defaultdict
#typing

#overrides

from parsimonious.expressions import Sequence, OneOf, Literal
from parsimonious.nodes import Node, NodeVisitor
from parsimonious.grammar import Grammar

# This is the base definition of the SQL grammar written in a simplified sort of
# EBNF notation. The notation here is of the form:
#    nonterminal = right hand side
# Rules that differ only in capitalization of keywords are mapped to the same action by
# the ``SqlVisitor``.  The nonterminal of the first rule is the starting symbol.
# In addition to the grammar here, we add ``col_ref``, ``table_name`` based on the tables
# that ``SqlTableContext`` is initialized with. ``number`` and ``string`` are initialized to
# be empty and later on updated based on the utterances.

SQL_GRAMMAR_STR = ur"""
    statement           = query ws ";" ws
    query               = (ws "(" ws "SELECT" ws distinct ws select_results ws "FROM" ws table_refs ws where_clause ws ")" ws) /
                          (ws "SELECT" ws distinct ws select_results ws "FROM" ws table_refs ws where_clause ws)
                        
    select_results      = col_refs / agg
    agg                 = agg_func ws "(" ws col_ref ws ")"
    agg_func            = "MIN" / "min" / "MAX" / "max" / "COUNT" / "count"
    col_refs            = (col_ref ws "," ws col_refs) / (col_ref)
    table_refs          = (table_name ws "," ws table_refs) / (table_name)
    where_clause        = ("WHERE" ws "(" ws conditions ws ")" ws) / ("WHERE" ws conditions ws)
    
    conditions          = (condition ws conj ws conditions) / 
                          (condition ws conj ws "(" ws conditions ws ")") /
                          ("(" ws conditions ws ")" ws conj ws conditions) /
                          ("(" ws conditions ws ")") /
                          ("not" ws conditions ws ) /
                          ("NOT" ws conditions ws ) /
                          condition
    condition           = in_clause / ternaryexpr / biexpr
    in_clause           = (ws col_ref ws "IN" ws query ws)
    biexpr              = ( col_ref ws binaryop ws value) / (value ws binaryop ws value) / ( col_ref ws "LIKE" ws string)
    binaryop            = "+" / "-" / "*" / "/" / "=" /
                          ">=" / "<=" / ">" / "<"  / "is" / "IS"
    ternaryexpr         = (col_ref ws "not" ws "BETWEEN" ws value ws "AND" ws value ws) /
                          (col_ref ws "NOT" ws "BETWEEN" ws value ws "AND" ws value ws) /
                          (col_ref ws "BETWEEN" ws value ws "AND" ws value ws)
    value               = ("not" ws pos_value) / ("NOT" ws pos_value) /(pos_value)
    pos_value           = ("ALL" ws query) / ("ANY" ws query) / number / boolean / col_ref / string / agg_results / "NULL"
    agg_results         = (ws "("  ws "SELECT" ws distinct ws agg ws "FROM" ws table_name ws where_clause ws ")" ws) /
                          (ws "SELECT" ws distinct ws agg ws "FROM" ws table_name ws where_clause ws)
    boolean             = "true" / "false"
    ws                  = ~"\s*"i
    conj                = "AND" / "OR" 
    distinct            = ("DISTINCT") / ("")
    number              =  ""
    string              =  ""
"""

KEYWORDS = [u'"SELECT"', u'"FROM"', u'"MIN"', u'"MAX"', u'"COUNT"', u'"WHERE"', u'"NOT"', u'"IN"', u'"LIKE"',
            u'"IS"', u'"BETWEEN"', u'"AND"', u'"ALL"', u'"ANY"', u'"NULL"', u'"OR"', u'"DISTINCT"']

def generate_one_of_string(nonterminal     , literals           )       :
    if literals:
        return  "\n{nonterminal} \t\t = " + u" / ".join([f'"{literal}"' for literal in literals])
    return  '\n{nonterminal} \t\t = ""'

def format_action(nonterminal     , right_hand_side     )       :
    if right_hand_side.upper() in KEYWORDS:
        right_hand_side = right_hand_side.upper()

    if nonterminal == u'string':
        return '{nonterminal} -> ["\'{right_hand_side}\'"]'

    elif nonterminal == u'number':
        return '{nonterminal} -> ["{right_hand_side}"]'

    else:
        right_hand_side = right_hand_side.lstrip(u"(").rstrip(u")")
        child_strings = [token for token in re.split(u" ws |ws | ws", right_hand_side) if token]
        child_strings = [tok.upper() if tok.upper() in KEYWORDS else tok for tok in child_strings]
        return "{nonterminal} -> [{', '.join(child_strings)}]"

class SqlTableContext(object):
    u"""
    A ``SqlTableContext`` represents the SQL context with grammar of SQL and the valid actions
    based on the schema of the tables that it represents.

    Parameters
    ----------
    tables: ``Dict[str, List[str]]``
        A dictionary representing the SQL tables in the dataset, the keys are the names of the tables
        that map to lists of the table's column names.
    """
    def __init__(self, tables                       = None)        :
        self.tables = tables
        self.grammar_str: unicode = self.initialize_grammar_str()
        self.grammar: Grammar = Grammar(self.grammar_str)
        self.valid_actions =  self.initialize_valid_actions()

    def initialize_valid_actions(self)                        :
        u"""
        We initialize the valid actions with the global actions. These include the
        valid actions that result from the grammar and also those that result from
        the tables provided. The keys represent the nonterminals in the grammar
        and the values are lists of the valid actions of that nonterminal.
        """
        valid_actions                      = defaultdict(set)

        for key in self.grammar:
            rhs = self.grammar[key]

            # Sequence represents a series of expressions that match pieces of the text in order.
            # Eg. A -> B C
            if isinstance(rhs, Sequence):
                valid_actions[key].add(format_action(key, u" ".join(rhs._unicode_members()))) # pylint: disable=protected-access

            # OneOf represents a series of expressions, one of which matches the text.
            # Eg. A -> B / C
            elif isinstance(rhs, OneOf):
                for option in rhs._unicode_members(): # pylint: disable=protected-access
                    valid_actions[key].add(format_action(key, option))

            # A string literal, eg. "A"
            elif isinstance(rhs, Literal):
                if rhs.literal != u"":
                    valid_actions[key].add(format_action(key, rhs.literal))
                else:
                    valid_actions[key] = set()

        valid_action_strings = dict((key, sorted(value)) for key, value in list(valid_actions.items()))
        return valid_action_strings

    def initialize_grammar_str(self):
        grammar_str = SQL_GRAMMAR_STR

        if self.tables:
            column_right_hand_sides = [u'"*"']
            for table, columns in list(self.tables.items()):
                column_right_hand_sides.extend([f'("{table}" ws "." ws "{column}")' for column in columns])
            grammar_str += u"\n      col_ref \t\t = " +\
                    u" / ".join(sorted(column_right_hand_sides, reverse=True))

            grammar_str += generate_one_of_string(u'table_name', sorted(list(self.tables.keys()), reverse=True))

        return grammar_str

class SqlVisitor(NodeVisitor):
    u"""
    ``SqlVisitor`` performs a depth-first traversal of the the AST. It takes the parse tree
    and gives us an action sequence that resulted in that parse. Since the visitor has mutable
    state, we define a new ``SqlVisitor`` for each query. To get the action sequence, we create
    a ``SqlVisitor`` and call parse on it, which returns a list of actions. Ex.

        sql_visitor = SqlVisitor(grammar_string)
        action_sequence = sql_visitor.parse(query)

    Parameters
    ----------
    grammar : ``Grammar``
        A Grammar object that we use to parse the text.
    """
    def __init__(self, grammar         )        :
        self.action_sequence =  []
        self.grammar: Grammar = grammar

    #overrides
    def generic_visit(self, node      , visited_children            )             :
        self.add_action(node)
        if node.expr.name == u'statement':
            return self.action_sequence
        return []

    def add_action(self, node      )        :
        u"""
        For each node, we accumulate the rules that generated its children in a list.
        """
        if node.expr.name and node.expr.name != u'ws':
            nonterminal = '{node.expr.name} -> '

            if isinstance(node.expr, Literal):
                right_hand_side = '["{node.text}"]'

            else:
                child_strings = []
                for child in node.__iter__():
                    if child.expr.name == u'ws':
                        continue
                    if child.expr.name != u'':
                        child_strings.append(child.expr.name)
                    else:
                        child_right_side_string = child.expr._as_rhs().lstrip(u"(").rstrip(u")") # pylint: disable=protected-access
                        child_right_side_list = [tok for tok in\
                                                 re.split(u" ws |ws | ws", child_right_side_string) if tok]
                        child_right_side_list = [tok.upper() if tok.upper() in KEYWORDS else tok\
                                                 for tok in child_right_side_list]
                        child_strings.extend(child_right_side_list)
                right_hand_side = u"[" + u", ".join(child_strings) + u"]"

            rule = nonterminal + right_hand_side
            self.action_sequence = [rule] + self.action_sequence





from __future__ import division
from __future__ import absolute_import
import torch
#overrides
from allennlp.modules.attention.legacy_attention import Attention


class CosineAttention(Attention):
    u"""
    Computes attention between a vector and a matrix using cosine similarity.
    """
    #overrides
    def _forward_internal(self, vector              , matrix              )                :
        a_norm = vector / (vector.norm(p=2, dim=-1, keepdim=True) + 1e-13)
        b_norm = matrix / (matrix.norm(p=2, dim=-1, keepdim=True) + 1e-13)
        return torch.bmm(a_norm.unsqueeze(dim=1), b_norm.transpose(-1, -2)).squeeze(1)

CosineAttention = Attention.register(u"cosine")(CosineAttention)



u"""
Functions and exceptions for checking that
AllenNLP and its models are configured correctly.
"""


from __future__ import absolute_import
import logging

from torch import cuda

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name


class ConfigurationError(Exception):
    u"""
    The exception raised by any AllenNLP object when it's misconfigured
    (e.g. missing properties, invalid properties, unknown properties).
    """

    def __init__(self, message):
        super(ConfigurationError, self).__init__()
        self.message = message

    def __str__(self):
        return repr(self.message)


def log_pytorch_version_info():
    import torch
    logger.info(u"Pytorch version: %s", torch.__version__)


def check_dimensions_match(dimension_1     ,
                           dimension_2     ,
                           dim_1_name     ,
                           dim_2_name     )        :
    if dimension_1 != dimension_2:
        raise ConfigurationError("{dim_1_name} must match {dim_2_name}, but got {dimension_1} "
                                 "and {dimension_2} instead")


def check_for_gpu(device_id     ):
    if device_id is not None and device_id >= cuda.device_count():
        raise ConfigurationError(u"Experiment specified a GPU but none is available;"
                                 u" if you want to run on CPU use the override"
                                 u" 'trainer.cuda_device=-1' in the json config file.")



# pylint: disable=no-self-use,invalid-name


from __future__ import with_statement
from __future__ import absolute_import
import pytest

from allennlp.common.configuration import configure, Config, BASE_CONFIG
from allennlp.common.testing import AllenNlpTestCase


class TestConfiguration(AllenNlpTestCase):
    def test_configure_top_level(self):
        config = configure()

        assert config == BASE_CONFIG

    def test_abstract_base_class(self):
        config = configure(u'allennlp.data.dataset_readers.dataset_reader.DatasetReader')

        assert isinstance(config, list)
        assert u'allennlp.data.dataset_readers.snli.SnliReader' in config

    def test_specific_subclass(self):
        config = configure(u'allennlp.data.dataset_readers.semantic_role_labeling.SrlReader')
        assert isinstance(config, Config)

        items = dict((item.name, item) for item in config.items)

        assert len(items) == 3

        assert u'token_indexers' in items
        token_indexers = items[u'token_indexers']
        assert token_indexers.default_value is None

        assert u'domain_identifier' in items
        domain_identifier = items[u'domain_identifier']
        assert domain_identifier.annotation == unicode
        assert domain_identifier.default_value is None

        assert u'lazy' in items
        lazy = items[u'lazy']
        assert lazy.annotation == bool
        assert not lazy.default_value

    def test_errors(self):
        with pytest.raises(ModuleNotFoundError):
            configure(u'allennlp.non_existent_module.SomeClass')

        with pytest.raises(AttributeError):
            configure(u'allennlp.data.dataset_readers.NonExistentDatasetReader')



# pylint: disable=invalid-name,no-self-use



from __future__ import with_statement
from __future__ import division
from __future__ import absolute_import
import argparse
#typing
import os
import shutil
import re

import pytest
import torch

from allennlp.common import Params
from allennlp.common.checks import ConfigurationError
from allennlp.common.testing import AllenNlpTestCase
from allennlp.commands.train import Train, train_model, train_model_from_args
from allennlp.data import DatasetReader, Instance
from io import open

SEQUENCE_TAGGING_DATA_PATH = unicode(AllenNlpTestCase.FIXTURES_ROOT / u'data' / u'sequence_tagging.tsv')

class TestTrain(AllenNlpTestCase):

    def test_train_model(self):
        params = lambda: Params({
                u"model": {
                        u"type": u"simple_tagger",
                        u"text_field_embedder": {
                                u"tokens": {
                                        u"type": u"embedding",
                                        u"embedding_dim": 5
                                }
                        },
                        u"encoder": {
                                u"type": u"lstm",
                                u"input_size": 5,
                                u"hidden_size": 7,
                                u"num_layers": 2
                        }
                },
                u"dataset_reader": {u"type": u"sequence_tagging"},
                u"train_data_path": SEQUENCE_TAGGING_DATA_PATH,
                u"validation_data_path": SEQUENCE_TAGGING_DATA_PATH,
                u"iterator": {u"type": u"basic", u"batch_size": 2},
                u"trainer": {
                        u"num_epochs": 2,
                        u"optimizer": u"adam"
                }
        })

        train_model(params(), serialization_dir=os.path.join(self.TEST_DIR, u'test_train_model'))

        # It's OK if serialization dir exists but is empty:
        serialization_dir2 = os.path.join(self.TEST_DIR, u'empty_directory')
        assert not os.path.exists(serialization_dir2)
        os.makedirs(serialization_dir2)
        train_model(params(), serialization_dir=serialization_dir2)

        # It's not OK if serialization dir exists and has junk in it non-empty:
        serialization_dir3 = os.path.join(self.TEST_DIR, u'non_empty_directory')
        assert not os.path.exists(serialization_dir3)
        os.makedirs(serialization_dir3)
        with open(os.path.join(serialization_dir3, u'README.md'), u'w') as f:
            f.write(u"TEST")

        with pytest.raises(ConfigurationError):
            train_model(params(), serialization_dir=serialization_dir3)

        # It's also not OK if serialization dir is a real serialization dir:
        with pytest.raises(ConfigurationError):
            train_model(params(), serialization_dir=os.path.join(self.TEST_DIR, u'test_train_model'))

    def test_error_is_throw_when_cuda_device_is_not_available(self):
        params = Params({
                u"model": {
                        u"type": u"simple_tagger",
                        u"text_field_embedder": {
                                u"tokens": {
                                        u"type": u"embedding",
                                        u"embedding_dim": 5
                                }
                        },
                        u"encoder": {
                                u"type": u"lstm",
                                u"input_size": 5,
                                u"hidden_size": 7,
                                u"num_layers": 2
                        }
                },
                u"dataset_reader": {u"type": u"sequence_tagging"},
                u"train_data_path": u'tests/fixtures/data/sequence_tagging.tsv',
                u"validation_data_path": u'tests/fixtures/data/sequence_tagging.tsv',
                u"iterator": {u"type": u"basic", u"batch_size": 2},
                u"trainer": {
                        u"num_epochs": 2,
                        u"cuda_device": torch.cuda.device_count(),
                        u"optimizer": u"adam"
                }
        })

        with pytest.raises(ConfigurationError,
                           message=u"Experiment specified a GPU but none is available;"
                                   u" if you want to run on CPU use the override"
                                   u" 'trainer.cuda_device=-1' in the json config file."):
            train_model(params, serialization_dir=os.path.join(self.TEST_DIR, u'test_train_model'))

    def test_train_with_test_set(self):
        params = Params({
                u"model": {
                        u"type": u"simple_tagger",
                        u"text_field_embedder": {
                                u"tokens": {
                                        u"type": u"embedding",
                                        u"embedding_dim": 5
                                }
                        },
                        u"encoder": {
                                u"type": u"lstm",
                                u"input_size": 5,
                                u"hidden_size": 7,
                                u"num_layers": 2
                        }
                },
                u"dataset_reader": {u"type": u"sequence_tagging"},
                u"train_data_path": SEQUENCE_TAGGING_DATA_PATH,
                u"test_data_path": SEQUENCE_TAGGING_DATA_PATH,
                u"validation_data_path": SEQUENCE_TAGGING_DATA_PATH,
                u"evaluate_on_test": True,
                u"iterator": {u"type": u"basic", u"batch_size": 2},
                u"trainer": {
                        u"num_epochs": 2,
                        u"optimizer": u"adam"
                }
        })

        train_model(params, serialization_dir=os.path.join(self.TEST_DIR, u'train_with_test_set'))

    def test_train_args(self):
        parser = argparse.ArgumentParser(description=u"Testing")
        subparsers = parser.add_subparsers(title=u'Commands', metavar=u'')
        Train().add_subparser(u'train', subparsers)

        for serialization_arg in [u"-s", u"--serialization-dir"]:
            raw_args = [u"train", u"path/to/params", serialization_arg, u"serialization_dir"]

            args = parser.parse_args(raw_args)

            assert args.func == train_model_from_args
            assert args.param_path == u"path/to/params"
            assert args.serialization_dir == u"serialization_dir"

        # config is required
        with self.assertRaises(SystemExit) as cm:  # pylint: disable=invalid-name
            args = parser.parse_args([u"train", u"-s", u"serialization_dir"])
            assert cm.exception.code == 2  # argparse code for incorrect usage

        # serialization dir is required
        with self.assertRaises(SystemExit) as cm:  # pylint: disable=invalid-name
            args = parser.parse_args([u"train", u"path/to/params"])
            assert cm.exception.code == 2  # argparse code for incorrect usage

class LazyFakeReader(DatasetReader):
    # pylint: disable=abstract-method
    def __init__(self)        :
        super(LazyFakeReader, self).__init__(lazy=True)
        self.reader = DatasetReader.from_params(Params({u'type': u'sequence_tagging'}))

    def _read(self, file_path     )                      :
        u"""
        Reads some data from the `file_path` and returns the instances.
        """
        return self.reader.read(file_path)


LazyFakeReader = DatasetReader.register(u'lazy-test')(LazyFakeReader)

class TestTrainOnLazyDataset(AllenNlpTestCase):
    def test_train_model(self):
        params = Params({
                u"model": {
                        u"type": u"simple_tagger",
                        u"text_field_embedder": {
                                u"tokens": {
                                        u"type": u"embedding",
                                        u"embedding_dim": 5
                                }
                        },
                        u"encoder": {
                                u"type": u"lstm",
                                u"input_size": 5,
                                u"hidden_size": 7,
                                u"num_layers": 2
                        }
                },
                u"dataset_reader": {u"type": u"lazy-test"},
                u"train_data_path": SEQUENCE_TAGGING_DATA_PATH,
                u"validation_data_path": SEQUENCE_TAGGING_DATA_PATH,
                u"iterator": {u"type": u"basic", u"batch_size": 2},
                u"trainer": {
                        u"num_epochs": 2,
                        u"optimizer": u"adam"
                }
        })

        train_model(params, serialization_dir=os.path.join(self.TEST_DIR, u'train_lazy_model'))

    def test_train_with_test_set(self):
        params = Params({
                u"model": {
                        u"type": u"simple_tagger",
                        u"text_field_embedder": {
                                u"tokens": {
                                        u"type": u"embedding",
                                        u"embedding_dim": 5
                                }
                        },
                        u"encoder": {
                                u"type": u"lstm",
                                u"input_size": 5,
                                u"hidden_size": 7,
                                u"num_layers": 2
                        }
                },
                u"dataset_reader": {u"type": u"lazy-test"},
                u"train_data_path": SEQUENCE_TAGGING_DATA_PATH,
                u"test_data_path": SEQUENCE_TAGGING_DATA_PATH,
                u"validation_data_path": SEQUENCE_TAGGING_DATA_PATH,
                u"evaluate_on_test": True,
                u"iterator": {u"type": u"basic", u"batch_size": 2},
                u"trainer": {
                        u"num_epochs": 2,
                        u"optimizer": u"adam"
                }
        })

        train_model(params, serialization_dir=os.path.join(self.TEST_DIR, u'lazy_test_set'))

    def test_train_nograd_regex(self):
        params_get = lambda: Params({
                u"model": {
                        u"type": u"simple_tagger",
                        u"text_field_embedder": {
                                u"tokens": {
                                        u"type": u"embedding",
                                        u"embedding_dim": 5
                                }
                        },
                        u"encoder": {
                                u"type": u"lstm",
                                u"input_size": 5,
                                u"hidden_size": 7,
                                u"num_layers": 2
                        }
                },
                u"dataset_reader": {u"type": u"sequence_tagging"},
                u"train_data_path": SEQUENCE_TAGGING_DATA_PATH,
                u"validation_data_path": SEQUENCE_TAGGING_DATA_PATH,
                u"iterator": {u"type": u"basic", u"batch_size": 2},
                u"trainer": {
                        u"num_epochs": 2,
                        u"optimizer": u"adam"
                }
        })
        serialization_dir = os.path.join(self.TEST_DIR, u'test_train_nograd')
        regex_lists = [[],
                       [u".*text_field_embedder.*"],
                       [u".*text_field_embedder.*", u".*encoder.*"]]
        for regex_list in regex_lists:
            params = params_get()
            params[u"trainer"][u"no_grad"] = regex_list
            shutil.rmtree(serialization_dir, ignore_errors=True)
            model = train_model(params, serialization_dir=serialization_dir)
            # If regex is matched, parameter name should have requires_grad False
            # Or else True
            for name, parameter in model.named_parameters():
                if any(re.search(regex, name) for regex in regex_list):
                    assert not parameter.requires_grad
                else:
                    assert parameter.requires_grad
        # If all parameters have requires_grad=False, then error.
        params = params_get()
        params[u"trainer"][u"no_grad"] = [u"*"]
        shutil.rmtree(serialization_dir, ignore_errors=True)
        with pytest.raises(Exception) as _:
            model = train_model(params, serialization_dir=serialization_dir)



# pylint: disable=no-self-use,invalid-name


from __future__ import with_statement
from __future__ import absolute_import
import numpy
import pytest
import torch

from allennlp.modules import SpanPruner
from allennlp.common.testing import AllenNlpTestCase
from allennlp.nn.util import batched_index_select

class TestSpanPruner(AllenNlpTestCase):
    def test_span_pruner_selects_top_scored_spans_and_respects_masking(self):
        # Really simple scorer - sum up the embedding_dim.
        scorer = lambda tensor: tensor.sum(-1).unsqueeze(-1)
        pruner = SpanPruner(scorer=scorer)

        spans = torch.randn([3, 4, 5]).clamp(min=0.0, max=1.0)
        spans[0, :2, :] = 1
        spans[1, 2:, :] = 1
        spans[2, 2:, :] = 1

        mask = torch.ones([3, 4])
        mask[1, 0] = 0
        mask[1, 3] = 0
        pruned_embeddings, pruned_mask, pruned_indices, pruned_scores = pruner(spans, mask, 2)

        # Second element in the batch would have indices 2, 3, but
        # 3 and 0 are masked, so instead it has 1, 2.
        numpy.testing.assert_array_equal(pruned_indices.data.numpy(), numpy.array([[0, 1],
                                                                                   [1, 2],
                                                                                   [2, 3]]))
        numpy.testing.assert_array_equal(pruned_mask.data.numpy(), numpy.ones([3, 2]))

        # embeddings should be the result of index_selecting the pruned_indices.
        correct_embeddings = batched_index_select(spans, pruned_indices)
        numpy.testing.assert_array_equal(correct_embeddings.data.numpy(),
                                         pruned_embeddings.data.numpy())
        # scores should be the sum of the correct embedding elements.
        numpy.testing.assert_array_equal(correct_embeddings.sum(-1).unsqueeze(-1).data.numpy(),
                                         pruned_scores.data.numpy())

    def test_span_scorer_raises_with_incorrect_scorer_spec(self):
        # Mis-configured scorer - doesn't produce a tensor with 1 as it's final dimension.
        scorer = lambda tensor: tensor.sum(-1)
        pruner = SpanPruner(scorer=scorer) # type: ignore
        spans = torch.randn([3, 4, 5]).clamp(min=0.0, max=1.0)
        mask = torch.ones([3, 4])

        with pytest.raises(ValueError):
            _ = pruner(spans, mask, 2)

    def test_span_scorer_works_for_completely_masked_rows(self):
        # Really simple scorer - sum up the embedding_dim.
        scorer = lambda tensor: tensor.sum(-1).unsqueeze(-1)
        pruner = SpanPruner(scorer=scorer) # type: ignore

        spans = torch.randn([3, 4, 5]).clamp(min=0.0, max=1.0)
        spans[0, :2, :] = 1
        spans[1, 2:, :] = 1
        spans[2, 2:, :] = 1

        mask = torch.ones([3, 4])
        mask[1, 0] = 0
        mask[1, 3] = 0
        mask[2, :] = 0 # fully masked last batch element.

        pruned_embeddings, pruned_mask, pruned_indices, pruned_scores = pruner(spans, mask, 2)

        # We can't check the last row here, because it's completely masked.
        # Instead we'll check that the scores for these elements are -inf.
        numpy.testing.assert_array_equal(pruned_indices[:2].data.numpy(), numpy.array([[0, 1],
                                                                                       [1, 2]]))
        numpy.testing.assert_array_equal(pruned_mask.data.numpy(), numpy.array([[1, 1],
                                                                                [1, 1],
                                                                                [0, 0]]))
        # embeddings should be the result of index_selecting the pruned_indices.
        correct_embeddings = batched_index_select(spans, pruned_indices)
        numpy.testing.assert_array_equal(correct_embeddings.data.numpy(),
                                         pruned_embeddings.data.numpy())
        # scores should be the sum of the correct embedding elements, with
        # masked elements equal to -inf.
        correct_scores = correct_embeddings.sum(-1).unsqueeze(-1).data.numpy()
        correct_scores[2, :] = float(u"-inf")
        numpy.testing.assert_array_equal(correct_scores,
                                         pruned_scores.data.numpy())



# pylint: disable=no-self-use,invalid-name

from __future__ import absolute_import
from collections import defaultdict

from allennlp.common.testing import AllenNlpTestCase
from allennlp.data import Token
from allennlp.data.token_indexers import SingleIdTokenIndexer


class TestSingleIdTokenIndexer(AllenNlpTestCase):
    def test_count_vocab_items_respects_casing(self):
        indexer = SingleIdTokenIndexer(u"words")
        counter = defaultdict(lambda: defaultdict(int))
        indexer.count_vocab_items(Token(u"Hello"), counter)
        indexer.count_vocab_items(Token(u"hello"), counter)
        assert counter[u"words"] == {u"hello": 1, u"Hello": 1}

        indexer = SingleIdTokenIndexer(u"words", lowercase_tokens=True)
        counter = defaultdict(lambda: defaultdict(int))
        indexer.count_vocab_items(Token(u"Hello"), counter)
        indexer.count_vocab_items(Token(u"hello"), counter)
        assert counter[u"words"] == {u"hello": 2}

    def test_as_array_produces_token_sequence(self):
        indexer = SingleIdTokenIndexer(u"words")
        padded_tokens = indexer.pad_token_sequence({u'key': [1, 2, 3, 4, 5]}, {u'key': 10}, {})
        assert padded_tokens == {u'key': [1, 2, 3, 4, 5, 0, 0, 0, 0, 0]}





from __future__ import division
from __future__ import absolute_import
from __future__ import print_function
import copy

from numpy.testing import assert_allclose
import torch

from allennlp.commands.train import train_model_from_file
from allennlp.common import Params
from allennlp.common.testing.test_case import AllenNlpTestCase
from allennlp.data import DataIterator, DatasetReader, Vocabulary
from allennlp.data.dataset import Batch
from allennlp.models import Model, load_archive
try:
    from itertools import izip
except:
    izip = zip



class ModelTestCase(AllenNlpTestCase):
    u"""
    A subclass of :class:`~allennlp.common.testing.test_case.AllenNlpTestCase`
    with added methods for testing :class:`~allennlp.models.model.Model` subclasses.
    """
    def set_up_model(self, param_file, dataset_file):
        # pylint: disable=attribute-defined-outside-init
        self.param_file = param_file
        params = Params.from_file(self.param_file)

        reader = DatasetReader.from_params(params[u'dataset_reader'])
        instances = reader.read(dataset_file)
        # Use parameters for vocabulary if they are present in the config file, so that choices like
        # "non_padded_namespaces", "min_count" etc. can be set if needed.
        if u'vocabulary' in params:
            vocab_params = params[u'vocabulary']
            vocab = Vocabulary.from_params(params=vocab_params, instances=instances)
        else:
            vocab = Vocabulary.from_instances(instances)
        self.vocab = vocab
        self.instances = instances
        self.model = Model.from_params(vocab=self.vocab, params=params[u'model'])

        # TODO(joelgrus) get rid of these
        # (a lot of the model tests use them, so they'll have to be changed)
        self.dataset = Batch(self.instances)
        self.dataset.index_instances(self.vocab)

    def ensure_model_can_train_save_and_load(self,
                                             param_file     ,
                                             tolerance        = 1e-4,
                                             cuda_device      = -1):
        save_dir = self.TEST_DIR / u"save_and_load_test"
        archive_file = save_dir / u"model.tar.gz"
        model = train_model_from_file(param_file, save_dir)
        loaded_model = load_archive(archive_file, cuda_device=cuda_device).model
        state_keys = list(model.state_dict().keys())
        loaded_state_keys = list(loaded_model.state_dict().keys())
        assert state_keys == loaded_state_keys
        # First we make sure that the state dict (the parameters) are the same for both models.
        for key in state_keys:
            assert_allclose(model.state_dict()[key].cpu().numpy(),
                            loaded_model.state_dict()[key].cpu().numpy(),
                            err_msg=key)
        params = Params.from_file(param_file)
        reader = DatasetReader.from_params(params[u'dataset_reader'])

        # Need to duplicate params because Iterator.from_params will consume.
        iterator_params = params[u'iterator']
        iterator_params2 = Params(copy.deepcopy(iterator_params.as_dict()))

        iterator = DataIterator.from_params(iterator_params)
        iterator2 = DataIterator.from_params(iterator_params2)

        # We'll check that even if we index the dataset with each model separately, we still get
        # the same result out.
        model_dataset = reader.read(params[u'validation_data_path'])
        iterator.index_with(model.vocab)
        model_batch = next(iterator(model_dataset, shuffle=False, cuda_device=cuda_device))

        loaded_dataset = reader.read(params[u'validation_data_path'])
        iterator2.index_with(loaded_model.vocab)
        loaded_batch = next(iterator2(loaded_dataset, shuffle=False, cuda_device=cuda_device))

        # Check gradients are None for non-trainable parameters and check that
        # trainable parameters receive some gradient if they are trainable.
        self.check_model_computes_gradients_correctly(model, model_batch)

        # The datasets themselves should be identical.
        assert list(model_batch.keys()) == list(loaded_batch.keys())
        for key in list(model_batch.keys()):
            self.assert_fields_equal(model_batch[key], loaded_batch[key], key, 1e-6)

        # Set eval mode, to turn off things like dropout, then get predictions.
        model.eval()
        loaded_model.eval()
        # Models with stateful RNNs need their states reset to have consistent
        # behavior after loading.
        for model_ in [model, loaded_model]:
            for module in model_.modules():
                if hasattr(module, u'stateful') and module.stateful:
                    module.reset_states()
        model_predictions = model(**model_batch)
        loaded_model_predictions = loaded_model(**loaded_batch)

        # Check loaded model's loss exists and we can compute gradients, for continuing training.
        loaded_model_loss = loaded_model_predictions[u"loss"]
        assert loaded_model_loss is not None
        loaded_model_loss.backward()

        # Both outputs should have the same keys and the values for these keys should be close.
        for key in list(model_predictions.keys()):
            self.assert_fields_equal(model_predictions[key],
                                     loaded_model_predictions[key],
                                     name=key,
                                     tolerance=tolerance)

        return model, loaded_model

    def assert_fields_equal(self, field1, field2, name     , tolerance        = 1e-6)        :
        if isinstance(field1, torch.Tensor):
            assert_allclose(field1.detach().cpu().numpy(),
                            field2.detach().cpu().numpy(),
                            rtol=tolerance,
                            err_msg=name)
        elif isinstance(field1, dict):
            assert list(field1.keys()) == list(field2.keys())
            for key in field1:
                self.assert_fields_equal(field1[key],
                                         field2[key],
                                         tolerance=tolerance,
                                         name=name + u'.' + unicode(key))
        elif isinstance(field1, (list, tuple)):
            assert len(field1) == len(field2)
            for i, (subfield1, subfield2) in enumerate(izip(field1, field2)):
                self.assert_fields_equal(subfield1,
                                         subfield2,
                                         tolerance=tolerance,
                                         name=name + "[{i}]")
        elif isinstance(field1, (float, int)):
            assert_allclose([field1], [field2], rtol=tolerance, err_msg=name)
        else:
            assert field1 == field2

    @staticmethod
    def check_model_computes_gradients_correctly(model, model_batch):
        model.zero_grad()
        result = model(**model_batch)
        result[u"loss"].backward()
        has_zero_or_none_grads = {}
        for name, parameter in model.named_parameters():
            zeros = torch.zeros(parameter.size())
            if parameter.requires_grad:

                if parameter.grad is None:
                    has_zero_or_none_grads[name] = u"No gradient computed (i.e parameter.grad is None)"

                elif parameter.grad.is_sparse or parameter.grad.data.is_sparse:
                    pass

                # Some parameters will only be partially updated,
                # like embeddings, so we just check that any gradient is non-zero.
                elif (parameter.grad.cpu() == zeros).all():
                    has_zero_or_none_grads[name] = "zeros with shape ({tuple(parameter.grad.size())})"
            else:
                assert parameter.grad is None

        if has_zero_or_none_grads:
            for name, grad in list(has_zero_or_none_grads.items()):
                print("Parameter: {name} had incorrect gradient: {grad}")
            raise Exception(u"Incorrect gradients found. See stdout for more info.")

    def ensure_batch_predictions_are_consistent(self):
        self.model.eval()
        single_predictions = []
        for i, instance in enumerate(self.instances):
            dataset = Batch([instance])
            tensors = dataset.as_tensor_dict(dataset.get_padding_lengths())
            result = self.model(**tensors)
            single_predictions.append(result)
        full_dataset = Batch(self.instances)
        batch_tensors = full_dataset.as_tensor_dict(full_dataset.get_padding_lengths())
        batch_predictions = self.model(**batch_tensors)
        for i, instance_predictions in enumerate(single_predictions):
            for key, single_predicted in list(instance_predictions.items()):
                tolerance = 1e-6
                if u'loss' in key:
                    # Loss is particularly unstable; we'll just be satisfied if everything else is
                    # close.
                    continue
                single_predicted = single_predicted[0]
                batch_predicted = batch_predictions[key][i]
                if isinstance(single_predicted, torch.Tensor):
                    if single_predicted.size() != batch_predicted.size():
                        slices = tuple(slice(0, size) for size in single_predicted.size())
                        batch_predicted = batch_predicted[slices]
                    assert_allclose(single_predicted.data.numpy(),
                                    batch_predicted.data.numpy(),
                                    atol=tolerance,
                                    err_msg=key)
                else:
                    assert single_predicted == batch_predicted, key




from __future__ import absolute_import
#overrides

from allennlp.common.util import get_spacy_model
from allennlp.common.util import JsonDict
from allennlp.data import DatasetReader, Instance
from allennlp.models import Model
from allennlp.predictors.predictor import Predictor


class CorefPredictor(Predictor):
    u"""
    Predictor for the :class:`~allennlp.models.coreference_resolution.CoreferenceResolver` model.
    """
    def __init__(self, model       , dataset_reader               )        :
        super(CorefPredictor, self).__init__(model, dataset_reader)

        # We have to use spacy to tokenise our document here, because we need
        # to also know sentence boundaries to propose valid mentions.
        self._spacy = get_spacy_model(u"en_core_web_sm", pos_tags=True, parse=True, ner=False)

    def predict(self, document     )            :
        u"""
        Predict the coreference clusters in the given document.

        .. code-block:: js

            {
            "document": [tokenised document text]
            "clusters":
              [
                [
                  [start_index, end_index],
                  [start_index, end_index]
                ],
                [
                  [start_index, end_index],
                  [start_index, end_index],
                  [start_index, end_index],
                ],
                ....
              ]
            }

        Parameters
        ----------
        document : ``str``
            A string representation of a document.

        Returns
        -------
        A dictionary representation of the predicted coreference clusters.
        """
        return self.predict_json({u"document" : document})

    #overrides
    def _json_to_instance(self, json_dict          )            :
        u"""
        Expects JSON that looks like ``{"document": "string of document text"}``
        """
        document = json_dict[u"document"]
        spacy_document = self._spacy(document)
        sentences = [[token.text for token in sentence] for sentence in spacy_document.sents]
        instance = self._dataset_reader.text_to_instance(sentences)
        return instance

CorefPredictor = Predictor.register(u"coreference-resolution")(CorefPredictor)





from __future__ import division
from __future__ import absolute_import
import torch
#overrides

from allennlp.modules.matrix_attention.matrix_attention import MatrixAttention


class CosineMatrixAttention(MatrixAttention):
    u"""
    Computes attention between every entry in matrix_1 with every entry in matrix_2 using cosine
    similarity.
    """

    #overrides
    def forward(self, matrix_1              , matrix_2              )                :
        a_norm = matrix_1 / (matrix_1.norm(p=2, dim=-1, keepdim=True) + 1e-13)
        b_norm = matrix_2 / (matrix_2.norm(p=2, dim=-1, keepdim=True) + 1e-13)
        return torch.bmm(a_norm, b_norm.transpose(-1, -2))

CosineMatrixAttention = MatrixAttention.register(u"cosine")(CosineMatrixAttention)



# pylint: disable=no-self-use,invalid-name


from __future__ import with_statement
from __future__ import absolute_import
from __future__ import print_function
from collections import defaultdict
#typing

import pytest
import numpy

from allennlp.data import Token, Vocabulary
from allennlp.data.fields import TextField
from allennlp.data.token_indexers import SingleIdTokenIndexer, TokenCharactersIndexer, TokenIndexer

from allennlp.common.testing import AllenNlpTestCase
from allennlp.common.checks import ConfigurationError
from allennlp.common.util import pad_sequence_to_length


class DictReturningTokenIndexer(TokenIndexer):
    u"""
    A stub TokenIndexer that returns multiple arrays of different lengths.
    """
    def count_vocab_items(self, token       , counter                           ):
        pass

    def tokens_to_indices(self, tokens             ,
                          vocabulary            ,
                          index_name     )                        : # pylint: disable=unused-argument
        return {
                u"token_ids": [10, 15] +\
                         [vocabulary.get_token_index(token.text, u'words') for token in tokens] +\
                         [25],
                u"additional_key": [22, 29]
        }

    def get_padding_token(self)       :
        return 0

    def get_padding_lengths(self, token     )                  :  # pylint: disable=unused-argument
        return {}

    def pad_token_sequence(self,
                           tokens                      ,
                           desired_num_tokens                ,
                           padding_lengths                )                        :  # pylint: disable=unused-argument
        return dict((key, pad_sequence_to_length(val, desired_num_tokens[key])) for key, val in list(tokens.items()))

    def get_keys(self, index_name     )             :
        # pylint: disable=unused-argument,no-self-use
        return [u"token_ids", u"additional_key"]


class TestTextField(AllenNlpTestCase):
    def setUp(self):
        self.vocab = Vocabulary()
        self.vocab.add_token_to_namespace(u"sentence", namespace=u'words')
        self.vocab.add_token_to_namespace(u"A", namespace=u'words')
        self.vocab.add_token_to_namespace(u"A", namespace=u'characters')
        self.vocab.add_token_to_namespace(u"s", namespace=u'characters')
        self.vocab.add_token_to_namespace(u"e", namespace=u'characters')
        self.vocab.add_token_to_namespace(u"n", namespace=u'characters')
        self.vocab.add_token_to_namespace(u"t", namespace=u'characters')
        self.vocab.add_token_to_namespace(u"c", namespace=u'characters')
        super(TestTextField, self).setUp()

    def test_field_counts_vocab_items_correctly(self):
        field = TextField([Token(t) for t in [u"This", u"is", u"a", u"sentence", u"."]],
                          token_indexers={u"words": SingleIdTokenIndexer(u"words")})
        namespace_token_counts = defaultdict(lambda: defaultdict(int))
        field.count_vocab_items(namespace_token_counts)

        assert namespace_token_counts[u"words"][u"This"] == 1
        assert namespace_token_counts[u"words"][u"is"] == 1
        assert namespace_token_counts[u"words"][u"a"] == 1
        assert namespace_token_counts[u"words"][u"sentence"] == 1
        assert namespace_token_counts[u"words"][u"."] == 1
        assert list(namespace_token_counts.keys()) == [u"words"]

        field = TextField([Token(t) for t in [u"This", u"is", u"a", u"sentence", u"."]],
                          token_indexers={u"characters": TokenCharactersIndexer(u"characters")})
        namespace_token_counts = defaultdict(lambda: defaultdict(int))
        field.count_vocab_items(namespace_token_counts)

        assert namespace_token_counts[u"characters"][u"T"] == 1
        assert namespace_token_counts[u"characters"][u"h"] == 1
        assert namespace_token_counts[u"characters"][u"i"] == 2
        assert namespace_token_counts[u"characters"][u"s"] == 3
        assert namespace_token_counts[u"characters"][u"a"] == 1
        assert namespace_token_counts[u"characters"][u"e"] == 3
        assert namespace_token_counts[u"characters"][u"n"] == 2
        assert namespace_token_counts[u"characters"][u"t"] == 1
        assert namespace_token_counts[u"characters"][u"c"] == 1
        assert namespace_token_counts[u"characters"][u"."] == 1
        assert list(namespace_token_counts.keys()) == [u"characters"]

        field = TextField([Token(t) for t in [u"This", u"is", u"a", u"sentence", u"."]],
                          token_indexers={u"words": SingleIdTokenIndexer(u"words"),
                                          u"characters": TokenCharactersIndexer(u"characters")})
        namespace_token_counts = defaultdict(lambda: defaultdict(int))
        field.count_vocab_items(namespace_token_counts)
        assert namespace_token_counts[u"characters"][u"T"] == 1
        assert namespace_token_counts[u"characters"][u"h"] == 1
        assert namespace_token_counts[u"characters"][u"i"] == 2
        assert namespace_token_counts[u"characters"][u"s"] == 3
        assert namespace_token_counts[u"characters"][u"a"] == 1
        assert namespace_token_counts[u"characters"][u"e"] == 3
        assert namespace_token_counts[u"characters"][u"n"] == 2
        assert namespace_token_counts[u"characters"][u"t"] == 1
        assert namespace_token_counts[u"characters"][u"c"] == 1
        assert namespace_token_counts[u"characters"][u"."] == 1
        assert namespace_token_counts[u"words"][u"This"] == 1
        assert namespace_token_counts[u"words"][u"is"] == 1
        assert namespace_token_counts[u"words"][u"a"] == 1
        assert namespace_token_counts[u"words"][u"sentence"] == 1
        assert namespace_token_counts[u"words"][u"."] == 1
        assert set(namespace_token_counts.keys()) == set([u"words", u"characters"])

    def test_index_converts_field_correctly(self):
        vocab = Vocabulary()
        sentence_index = vocab.add_token_to_namespace(u"sentence", namespace=u'words')
        capital_a_index = vocab.add_token_to_namespace(u"A", namespace=u'words')
        capital_a_char_index = vocab.add_token_to_namespace(u"A", namespace=u'characters')
        s_index = vocab.add_token_to_namespace(u"s", namespace=u'characters')
        e_index = vocab.add_token_to_namespace(u"e", namespace=u'characters')
        n_index = vocab.add_token_to_namespace(u"n", namespace=u'characters')
        t_index = vocab.add_token_to_namespace(u"t", namespace=u'characters')
        c_index = vocab.add_token_to_namespace(u"c", namespace=u'characters')

        field = TextField([Token(t) for t in [u"A", u"sentence"]],
                          {u"words": SingleIdTokenIndexer(namespace=u"words")})
        field.index(vocab)
        # pylint: disable=protected-access
        assert field._indexed_tokens[u"words"] == [capital_a_index, sentence_index]

        field1 = TextField([Token(t) for t in [u"A", u"sentence"]],
                           {u"characters": TokenCharactersIndexer(namespace=u"characters")})
        field1.index(vocab)
        assert field1._indexed_tokens[u"characters"] == [[capital_a_char_index],
                                                        [s_index, e_index, n_index, t_index,
                                                         e_index, n_index, c_index, e_index]]
        field2 = TextField([Token(t) for t in [u"A", u"sentence"]],
                           token_indexers={u"words": SingleIdTokenIndexer(namespace=u"words"),
                                           u"characters": TokenCharactersIndexer(namespace=u"characters")})
        field2.index(vocab)
        assert field2._indexed_tokens[u"words"] == [capital_a_index, sentence_index]
        assert field2._indexed_tokens[u"characters"] == [[capital_a_char_index],
                                                        [s_index, e_index, n_index, t_index,
                                                         e_index, n_index, c_index, e_index]]
        # pylint: enable=protected-access

    def test_get_padding_lengths_raises_if_no_indexed_tokens(self):

        field = TextField([Token(t) for t in [u"This", u"is", u"a", u"sentence", u"."]],
                          token_indexers={u"words": SingleIdTokenIndexer(u"words")})
        with pytest.raises(ConfigurationError):
            field.get_padding_lengths()

    def test_padding_lengths_are_computed_correctly(self):
        field = TextField([Token(t) for t in [u"This", u"is", u"a", u"sentence", u"."]],
                          token_indexers={u"words": SingleIdTokenIndexer(u"words")})
        field.index(self.vocab)
        padding_lengths = field.get_padding_lengths()
        assert padding_lengths == {u"num_tokens": 5}

        field = TextField([Token(t) for t in [u"This", u"is", u"a", u"sentence", u"."]],
                          token_indexers={u"characters": TokenCharactersIndexer(u"characters")})
        field.index(self.vocab)
        padding_lengths = field.get_padding_lengths()
        assert padding_lengths == {u"num_tokens": 5, u"num_token_characters": 8}

        field = TextField([Token(t) for t in [u"This", u"is", u"a", u"sentence", u"."]],
                          token_indexers={u"characters": TokenCharactersIndexer(u"characters"),
                                          u"words": SingleIdTokenIndexer(u"words")})
        field.index(self.vocab)
        padding_lengths = field.get_padding_lengths()
        assert padding_lengths == {u"num_tokens": 5, u"num_token_characters": 8}

    def test_as_tensor_handles_words(self):
        field = TextField([Token(t) for t in [u"This", u"is", u"a", u"sentence", u"."]],
                          token_indexers={u"words": SingleIdTokenIndexer(u"words")})
        field.index(self.vocab)
        padding_lengths = field.get_padding_lengths()
        tensor_dict = field.as_tensor(padding_lengths)
        numpy.testing.assert_array_almost_equal(tensor_dict[u"words"].detach().cpu().numpy(),
                                                numpy.array([1, 1, 1, 2, 1]))

    def test_as_tensor_handles_longer_lengths(self):
        field = TextField([Token(t) for t in [u"This", u"is", u"a", u"sentence", u"."]],
                          token_indexers={u"words": SingleIdTokenIndexer(u"words")})
        field.index(self.vocab)
        padding_lengths = field.get_padding_lengths()
        padding_lengths[u"num_tokens"] = 10
        tensor_dict = field.as_tensor(padding_lengths)
        numpy.testing.assert_array_almost_equal(tensor_dict[u"words"].detach().cpu().numpy(),
                                                numpy.array([1, 1, 1, 2, 1, 0, 0, 0, 0, 0]))

    def test_as_tensor_handles_characters(self):
        field = TextField([Token(t) for t in [u"This", u"is", u"a", u"sentence", u"."]],
                          token_indexers={u"characters": TokenCharactersIndexer(u"characters")})
        field.index(self.vocab)
        padding_lengths = field.get_padding_lengths()
        tensor_dict = field.as_tensor(padding_lengths)
        expected_character_array = numpy.array([[1, 1, 1, 3, 0, 0, 0, 0],
                                                [1, 3, 0, 0, 0, 0, 0, 0],
                                                [1, 0, 0, 0, 0, 0, 0, 0],
                                                [3, 4, 5, 6, 4, 5, 7, 4],
                                                [1, 0, 0, 0, 0, 0, 0, 0]])
        numpy.testing.assert_array_almost_equal(tensor_dict[u"characters"].detach().cpu().numpy(),
                                                expected_character_array)

    def test_as_tensor_handles_words_and_characters_with_longer_lengths(self):
        field = TextField([Token(t) for t in [u"a", u"sentence", u"."]],
                          token_indexers={u"words": SingleIdTokenIndexer(u"words"),
                                          u"characters": TokenCharactersIndexer(u"characters")})
        field.index(self.vocab)
        padding_lengths = field.get_padding_lengths()
        padding_lengths[u"num_tokens"] = 5
        padding_lengths[u"num_token_characters"] = 10
        tensor_dict = field.as_tensor(padding_lengths)

        numpy.testing.assert_array_almost_equal(tensor_dict[u"words"].detach().cpu().numpy(),
                                                numpy.array([1, 2, 1, 0, 0]))
        numpy.testing.assert_array_almost_equal(tensor_dict[u"characters"].detach().cpu().numpy(),
                                                numpy.array([[1, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                                                             [3, 4, 5, 6, 4, 5, 7, 4, 0, 0],
                                                             [1, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                                                             [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                                                             [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]]))

    def test_printing_doesnt_crash(self):
        field = TextField([Token(t) for t in [u"A", u"sentence"]],
                          {u"words": SingleIdTokenIndexer(namespace=u"words")})
        print(field)

    def test_token_embedder_returns_dict(self):
        field = TextField([Token(t) for t in [u"A", u"sentence"]],
                          token_indexers={u"field_with_dict": DictReturningTokenIndexer(),
                                          u"words": SingleIdTokenIndexer(u"words"),
                                          u"characters": TokenCharactersIndexer(u"characters")})
        field.index(self.vocab)
        padding_lengths = field.get_padding_lengths()
        assert padding_lengths == {
                u'token_ids': 5,
                u'additional_key': 2,
                u'words': 2,
                u'characters': 2,
                u'num_token_characters': 8
        }
        padding_lengths[u'token_ids'] = 7
        padding_lengths[u'additional_key'] = 3
        padding_lengths[u'words'] = 4
        padding_lengths[u'characters'] = 4
        tensors = field.as_tensor(padding_lengths)
        assert list(tensors[u'token_ids'].shape) == [7]
        assert list(tensors[u'additional_key'].shape) == [3]
        assert list(tensors[u'words'].shape) == [4]
        assert list(tensors[u'characters'].shape) == [4, 8]





from __future__ import with_statement
from __future__ import absolute_import
import json
#typing
import logging

#overrides
from parsimonious.exceptions import ParseError

from allennlp.common.file_utils import cached_path
from allennlp.data.dataset_readers.dataset_reader import DatasetReader
from allennlp.data.fields import Field, ArrayField, ListField, IndexField,\
        ProductionRuleField, TextField, MetadataField
from allennlp.data.instance import Instance
from allennlp.data.token_indexers import SingleIdTokenIndexer, TokenIndexer
from allennlp.data.tokenizers import Tokenizer, WordTokenizer
from allennlp.data.tokenizers.word_splitter import SpacyWordSplitter

from allennlp.semparse.worlds.atis_world import AtisWorld
from io import open

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name

def _lazy_parse(text     ):
    for interaction in text.split(u"\n"):
        if interaction:
            yield json.loads(interaction)

class AtisDatasetReader(DatasetReader):
    # pylint: disable=line-too-long
    u"""
    This ``DatasetReader`` takes json files and converts them into ``Instances`` for the
    ``AtisSemanticParser``.

    Each line in the file is a JSON object that represent an interaction in the ATIS dataset
    that has the following keys and values:
    ```
    "id": The original filepath in the LDC corpus
    "interaction": <list where each element represents a turn in the interaction>
    "scenario": A code that refers to the scenario that served as the prompt for this interaction
    "ut_date": Date of the interaction
    "zc09_path": Path that was used in the original paper `Learning Context-Dependent Mappings from
    Sentences to Logical Form
    <https://www.semanticscholar.org/paper/Learning-Context-Dependent-Mappings-from-Sentences-Zettlemoyer-Collins/44a8fcee0741139fa15862dc4b6ce1e11444878f>'_ by Zettlemoyer and Collins (ACL/IJCNLP 2009)
    ```

    Each element in the ``interaction`` list has the following keys and values:
    ```
    "utterance": Natural language input
    "sql": A list of SQL queries that the utterance maps to, it could be multiple SQL queries
    or none at all.
    ```

    Parameters
    ----------
    token_indexers : ``Dict[str, TokenIndexer]``, optional
        Token indexers for the utterances. Will default to ``{"tokens": SingleIdTokenIndexer()}``.
    lazy : ``bool`` (optional, default=False)
        Passed to ``DatasetReader``.  If this is ``True``, training will start sooner, but will
        take longer per batch.
    tokenizer : ``Tokenizer``, optional
        Tokenizer to use for the utterances. Will default to ``WordTokenizer()`` with Spacy's tagger
        enabled.
    """
    def __init__(self,
                 token_indexers                          = None,
                 lazy       = False,
                 tokenizer            = None)        :
        super(AtisDatasetReader, self).__init__(lazy)
        self._token_indexers = token_indexers or {u'tokens': SingleIdTokenIndexer()}
        self._tokenizer = tokenizer or WordTokenizer(SpacyWordSplitter(pos_tags=True))


    #overrides
    def _read(self, file_path     ):
        # if `file_path` is a URL, redirect to the cache
        file_path = cached_path(file_path)

        with open(file_path) as atis_file:
            logger.info(u"Reading ATIS instances from dataset at : %s", file_path)
            for line in _lazy_parse(atis_file.read()):
                utterances = []
                for current_interaction in line[u'interaction']:
                    if not current_interaction[u'utterance']:
                        continue
                    utterances.append(current_interaction[u'utterance'])
                    instance = self.text_to_instance(utterances, current_interaction[u'sql'])
                    if not instance:
                        continue
                    yield instance

    #overrides
    def text_to_instance(self,  # type: ignore
                         utterances           ,
                         sql_query      = None)            :
        # pylint: disable=arguments-differ
        u"""
        Parameters
        ----------
        utterances: ``List[str]``, required.
            List of utterances in the interaction, the last element is the current utterance.
        sql_query: ``str``, optional
            The SQL query, given as label during training or validation.
        """
        utterance = utterances[-1]
        action_sequence            = []

        if not utterance:
            return None

        world = AtisWorld(utterances)

        if sql_query:
            try:
                action_sequence = world.get_action_sequence(sql_query)
            except ParseError:
                logger.debug('Parsing error')

        tokenized_utterance = self._tokenizer.tokenize(utterance.lower())
        utterance_field = TextField(tokenized_utterance, self._token_indexers)

        production_rule_fields              = []

        for production_rule in world.all_possible_actions():
            lhs, _ = production_rule.split(u' ->')
            is_global_rule = not lhs in [u'number', u'string']
            # The whitespaces are not semantically meaningful, so we filter them out.
            production_rule = u' '.join([token for token in production_rule.split(u' ') if token != u'ws'])
            field = ProductionRuleField(production_rule, is_global_rule)
            production_rule_fields.append(field)

        action_field = ListField(production_rule_fields)
        action_map = dict((action.rule, i) # type: ignore
                      for i, action in enumerate(action_field.field_list))
        index_fields              = []
        world_field = MetadataField(world)
        fields = {u'utterance' : utterance_field,
                  u'actions' : action_field,
                  u'world' : world_field,
                  u'linking_scores' : ArrayField(world.linking_scores)}

        if sql_query:
            if action_sequence:
                for production_rule in action_sequence:
                    index_fields.append(IndexField(action_map[production_rule], action_field))

                action_sequence_field              = []
                action_sequence_field.append(ListField(index_fields))
                fields[u'target_action_sequence'] = ListField(action_sequence_field)
            else:
                # If we are given a SQL query, but we are unable to parse it, then we will skip it.
                return None

        return Instance(fields)

AtisDatasetReader = DatasetReader.register(u"atis")(AtisDatasetReader)



# pylint: disable=no-self-use


from __future__ import division
from __future__ import absolute_import
import os
import pytest

from allennlp.common.testing import ModelTestCase
from allennlp.training.metrics.wikitables_accuracy import SEMPRE_ABBREVIATIONS_PATH, SEMPRE_GRAMMAR_PATH

class WikiTablesErmSemanticParserTest(ModelTestCase):
    def setUp(self):
        self.should_remove_sempre_abbreviations = not os.path.exists(SEMPRE_ABBREVIATIONS_PATH)
        self.should_remove_sempre_grammar = not os.path.exists(SEMPRE_GRAMMAR_PATH)

        # The model tests are run with respect to the module root, so check if abbreviations
        # and grammar already exist there (since we want to clean up module root after test)
        self.module_root_abbreviations_path = self.MODULE_ROOT / u"data" / u"abbreviations.tsv"
        self.module_root_grammar_path = self.MODULE_ROOT / u"data" / u"grow.grammar"
        self.should_remove_root_sempre_abbreviations = not os.path.exists(self.module_root_abbreviations_path)
        self.should_remove_root_sempre_grammar = not os.path.exists(self.module_root_grammar_path)

        super(WikiTablesErmSemanticParserTest, self).setUp()
        self.set_up_model(unicode(self.FIXTURES_ROOT / u"semantic_parsing" / u"wikitables" / u"experiment-erm.json"),
                          unicode(self.FIXTURES_ROOT / u"data" / u"wikitables" / u"sample_data.examples"))

    def tearDown(self):
        super(WikiTablesErmSemanticParserTest, self).tearDown()
        # We don't want to leave generated files around just from running tests...
        if self.should_remove_sempre_abbreviations and os.path.exists(SEMPRE_ABBREVIATIONS_PATH):
            os.remove(SEMPRE_ABBREVIATIONS_PATH)
        if self.should_remove_sempre_grammar and os.path.exists(SEMPRE_GRAMMAR_PATH):
            os.remove(SEMPRE_GRAMMAR_PATH)
        if self.should_remove_root_sempre_abbreviations and os.path.exists(self.module_root_abbreviations_path):
            os.remove(self.module_root_abbreviations_path)
        if self.should_remove_root_sempre_grammar and os.path.exists(self.module_root_grammar_path):
            os.remove(self.module_root_grammar_path)

    def test_model_can_train_save_and_load(self):
        self.ensure_model_can_train_save_and_load(self.param_file)

WikiTablesErmSemanticParserTest = pytest.mark.java(WikiTablesErmSemanticParserTest)



# pylint: disable=no-self-use,invalid-name


from __future__ import division
from __future__ import absolute_import
import pytest

from allennlp.common import Params
from allennlp.common.util import ensure_list
from allennlp.data.dataset_readers import TriviaQaReader
from allennlp.common.testing import AllenNlpTestCase

class TestTriviaQaReader(object):
    @pytest.mark.parametrize(u"lazy", (True, False))
    def test_read(self, lazy):
        params = Params({
                u'base_tarball_path': unicode(AllenNlpTestCase.FIXTURES_ROOT / u'data' / u'triviaqa-sample.tgz'),
                u'lazy': lazy
                })
        reader = TriviaQaReader.from_params(params)
        instances = reader.read(u'web-train.json')
        instances = ensure_list(instances)
        assert len(instances) == 3

        assert [t.text for t in instances[0].fields[u"question"].tokens[:3]] == [u"Which", u"American", u"-"]
        assert [t.text for t in instances[0].fields[u"passage"].tokens[:3]] == [u"The", u"Nobel", u"Prize"]
        url = u"http://www.nobelprize.org/nobel_prizes/literature/laureates/1930/"
        assert [t.text for t in instances[0].fields[u"passage"].tokens[-3:]] == [u"<", url, u">"]
        assert instances[0].fields[u"span_start"].sequence_index == 12
        assert instances[0].fields[u"span_end"].sequence_index == 13

        assert [t.text for t in instances[1].fields[u"question"].tokens[:3]] == [u"Which", u"American", u"-"]
        assert [t.text for t in instances[1].fields[u"passage"].tokens[:3]] == [u"Why", u"Do", u"n’t"]
        assert [t.text for t in instances[1].fields[u"passage"].tokens[-3:]] == [u"adults", u",", u"and"]
        assert instances[1].fields[u"span_start"].sequence_index == 38
        assert instances[1].fields[u"span_end"].sequence_index == 39

        assert [t.text for t in instances[2].fields[u"question"].tokens[:3]] == [u"Where", u"in", u"England"]
        assert [t.text for t in instances[2].fields[u"passage"].tokens[:3]] == [u"Judi", u"Dench", u"-"]
        assert [t.text for t in instances[2].fields[u"passage"].tokens[-3:]] == [u")", u"(", u"special"]
        assert instances[2].fields[u"span_start"].sequence_index == 16
        assert instances[2].fields[u"span_end"].sequence_index == 16



# pylint: disable=no-self-use,invalid-name,protected-access



from __future__ import with_statement
from __future__ import absolute_import
import torch
from allennlp.common.checks import ConfigurationError
from allennlp.training.optimizers import Optimizer
from allennlp.common.testing import AllenNlpTestCase
from allennlp.training.learning_rate_schedulers import LearningRateScheduler
from allennlp.common.params import Params


class LearningRateSchedulersTest(AllenNlpTestCase):

    def test_reduce_on_plateau_error_throw_when_no_metrics_exist(self):
        model = torch.nn.Sequential(torch.nn.Linear(10, 10))
        with self.assertRaises(ConfigurationError) as context:
            LearningRateScheduler.from_params(Optimizer.from_params(model.named_parameters(),
                                                                    Params({u"type": u"adam"})),
                                              Params({u"type": u"reduce_on_plateau"})).step(None, None)

        self.assertTrue(
                u'The reduce_on_plateau learning rate scheduler requires a validation metric'
                in unicode(context.exception))

    def test_reduce_on_plateau_works_when_metrics_exist(self):
        model = torch.nn.Sequential(torch.nn.Linear(10, 10))
        LearningRateScheduler.from_params(Optimizer.from_params(model.named_parameters(),
                                                                Params({u"type": u"adam"})),
                                          Params({u"type": u"reduce_on_plateau"})).step(10, None)

    def test_no_metric_wrapper_can_support_none_for_metrics(self):
        model = torch.nn.Sequential(torch.nn.Linear(10, 10))
        lrs = LearningRateScheduler.from_params(Optimizer.from_params(model.named_parameters(),
                                                                      Params({u"type": u"adam"})),
                                                Params({u"type": u"step", u"step_size": 1}))
        lrs.step(None, None)

    def test_noam_learning_rate_schedule_does_not_crash(self):
        model = torch.nn.Sequential(torch.nn.Linear(10, 10))
        lrs = LearningRateScheduler.from_params(Optimizer.from_params(model.named_parameters(),
                                                                      Params({u"type": u"adam"})),
                                                Params({u"type": u"noam", u"model_size": 10, u"warmup_steps": 2000}))
        lrs.step(None)
        lrs.step_batch(None)



# python
#
# See the accompanying LICENSE file.
#
# Work out how much coverage we actually have

import glob
import os
import sys

out=sys.stdout.write


def output(filename, percent, total):
    # Python bug "% 3.2f" doesn't behave correctly (100.00 is formatted with leading space!)
    op=["%-40s" % (filename,),
        "% 3.2f%%" % (percent,),
        " of % 6d" % (total,)]
    if percent==100:
        op[1]="100.00%"
    out("".join(op)+"\n")


linesexecuted=0
linestotal=0

names=glob.glob("src/*.c.gcov")
names.sort()

for f in names:
    if f.startswith("sqlite3"):
        continue
    fileexec=0
    filetotal=0
    for line in open(f, "rtU"):
        if ":" not in line: continue
        line=line.split(":", 1)[0].strip()
        if line=="-":
            continue
        if line!="#####":
            linesexecuted+=1
            fileexec+=1
        linestotal+=1
        filetotal+=1
    n=os.path.splitext(f)[0]
    output(n, fileexec*100.0/filetotal, filetotal)

out("\n")
output("Total", linesexecuted*100.0/(linestotal if linestotal else 1), linestotal)



from __future__ import absolute_import
#typing

import torch

from allennlp.semparse.worlds import WikiTablesWorld
from allennlp.data.fields.production_rule_field import ProductionRuleArray
from allennlp.nn.decoding import DecoderState, GrammarState, RnnState, ChecklistState
try:
    from itertools import izip
except:
    izip = zip


# This syntax is pretty weird and ugly, but it's necessary to make mypy happy with the API that
# we've defined.  We're using generics to make the type of `combine_states` come out right.  See
# the note in `nn.decoding.decoder_state.py` for a little more detail.
class WikiTablesDecoderState(DecoderState[u'WikiTablesDecoderState']):
    u"""
    Parameters
    ----------
    batch_indices : ``List[int]``
        Passed to super class; see docs there.
    action_history : ``List[List[int]]``
        Passed to super class; see docs there.
    score : ``List[torch.Tensor]``
        Passed to super class; see docs there.
    rnn_state : ``List[RnnState]``
        An ``RnnState`` for every group element.  This keeps track of the current decoder hidden
        state, the previous decoder output, the output from the encoder (for computing attentions),
        and other things that are typical seq2seq decoder state things.
    grammar_state : ``List[GrammarState]``
        This hold the current grammar state for each element of the group.  The ``GrammarState``
        keeps track of which actions are currently valid.
    action_embeddings : ``torch.Tensor``
        The global action embeddings tensor.  Has shape ``(num_global_embeddable_actions,
        action_embedding_dim)``.
    output_action_embeddings : ``torch.Tensor``
        The global output action embeddings tensor.  Has shape ``(num_global_embeddable_actions,
        action_embedding_dim)``.
    action_biases : ``torch.Tensor``
        A vector of biases for each action.  Has shape ``(num_global_embeddable_actions, 1)``.
    action_indices : ``Dict[Tuple[int, int], int]``
        A mapping from ``(batch_index, action_index)`` to ``global_action_index``.
    possible_actions : ``List[List[ProductionRuleArray]]``
        The list of all possible actions that was passed to ``model.forward()``.  We need this so
        we can recover production strings, which we need to update grammar states.
    flattened_linking_scores : ``torch.FloatTensor``
        Linking scores between table entities and question tokens.  The unflattened version has
        shape ``(batch_size, num_entities, num_question_tokens)``, though this version is flattened
        to have shape ``(batch_size * num_entities, num_question_tokens)``, for easier lookups with
        ``index_select``.
    actions_to_entities : ``Dict[Tuple[int, int], int]``
        A mapping from ``(batch_index, action_index)`` to ``batch_size * num_entities``, for
        actions that are terminal entity productions.
    entity_types : ``Dict[int, int]``
        A mapping from flattened entity indices (same as the `values` in the
        ``actions_to_entities`` dictionary) to entity type indices.  This represents what type each
        entity has, which we will use for getting type embeddings in certain circumstances.
    world : ``List[WikiTablesWorld]``, optional (default=None)
        The worlds corresponding to elements in the batch. We store them here because they're required
        for executing logical forms to determine costs while training, if we're learning to search.
        Otherwise, they're not required. Note that the worlds are batched, and they will be passed
        around unchanged during the decoding process.
    example_lisp_string : ``List[str]``, optional (default=None)
        The lisp strings that come from example files. They're also required for evaluating logical
        forms only if we're learning to search. These too are batched, and will be passed around
        unchanged.
    checklist_state : ``List[ChecklistState]``, optional (default=None)
        If you are using this state within a parser being trained for coverage, we need to store a
        ``ChecklistState`` which keeps track of the coverage information. Not needed if you are
        using a non-coverage based training algorithm.
    """
    def __init__(self,
                 batch_indices           ,
                 action_history                 ,
                 score                    ,
                 rnn_state                ,
                 grammar_state                    ,
                 action_embeddings              ,
                 output_action_embeddings              ,
                 action_biases              ,
                 action_indices                            ,
                 possible_actions                                 ,
                 flattened_linking_scores                   ,
                 actions_to_entities                            ,
                 entity_types                ,
                 world                        = None,
                 example_lisp_string            = None,
                 checklist_state                       = None,
                 debug_info       = None)        :
        super(WikiTablesDecoderState, self).__init__(batch_indices, action_history, score)
        self.rnn_state = rnn_state
        self.grammar_state = grammar_state
        self.action_embeddings = action_embeddings
        self.output_action_embeddings = output_action_embeddings
        self.action_biases = action_biases
        self.action_indices = action_indices
        # This is just the reverse of action indices.
        self.global_to_batch_action_indices =  {}
        for (batch_index, batch_action_index), global_index in action_indices.items():
            if global_index == -1:
                # The batch_action_index here corresponds to a linked action. We don't need to map
                # it.
                continue
            self.global_to_batch_action_indices[(batch_index, global_index)] = batch_action_index
        self.possible_actions = possible_actions
        self.flattened_linking_scores = flattened_linking_scores
        self.actions_to_entities = actions_to_entities
        self.entity_types = entity_types
        self.world = world
        self.example_lisp_string = example_lisp_string
        # Converting None to a list of Nones of appropriate size to avoid checking for None in all
        # state operations.
        self.checklist_state = checklist_state if checklist_state is not None else [None for _ in
                                                                                    batch_indices]
        self.debug_info = debug_info

    def print_action_history(self, group_index      = None)        :
        scores = self.score if group_index is None else [self.score[group_index]]
        batch_indices = self.batch_indices if group_index is None else [self.batch_indices[group_index]]
        histories = self.action_history if group_index is None else [self.action_history[group_index]]
        for score, batch_index, action_history in izip(scores, batch_indices, histories):
            print u'  ', score.detach().cpu().numpy()[0],
                  [self.possible_actions[batch_index][action][0] for action in action_history]

    def get_valid_actions(self)                   :
        u"""
        Returns a list of valid actions for each element of the group.
        """
        return [state.get_valid_actions() for state in self.grammar_state]

    def is_finished(self)        :
        if len(self.batch_indices) != 1:
            raise RuntimeError(u"is_finished() is only defined with a group_size of 1")
        return self.grammar_state[0].is_finished()

    @classmethod
    def combine_states(cls, states                                )                            :
        batch_indices = [batch_index for state in states for batch_index in state.batch_indices]
        action_histories = [action_history for state in states for action_history in state.action_history]
        scores = [score for state in states for score in state.score]
        rnn_states = [rnn_state for state in states for rnn_state in state.rnn_state]
        grammar_states = [grammar_state for state in states for grammar_state in state.grammar_state]
        checklist_states = [checklist_state for state in states for checklist_state in state.checklist_state]
        if states[0].debug_info is not None:
            debug_info = [debug_info for state in states for debug_info in state.debug_info]
        else:
            debug_info = None
        return WikiTablesDecoderState(batch_indices=batch_indices,
                                      action_history=action_histories,
                                      score=scores,
                                      rnn_state=rnn_states,
                                      grammar_state=grammar_states,
                                      action_embeddings=states[0].action_embeddings,
                                      output_action_embeddings=states[0].output_action_embeddings,
                                      action_biases=states[0].action_biases,
                                      action_indices=states[0].action_indices,
                                      possible_actions=states[0].possible_actions,
                                      flattened_linking_scores=states[0].flattened_linking_scores,
                                      actions_to_entities=states[0].actions_to_entities,
                                      entity_types=states[0].entity_types,
                                      world=states[0].world,
                                      example_lisp_string=states[0].example_lisp_string,
                                      checklist_state=checklist_states,
                                      debug_info=debug_info)



# pylint: disable=invalid-name,no-self-use,protected-access

from __future__ import absolute_import
import torch

from allennlp.common.testing import AllenNlpTestCase
from allennlp.nn.decoding import util


class TestNnDecodingUtil(AllenNlpTestCase):
    def test_create_allowed_transitions(self):
        targets = torch.autograd.Variable(torch.Tensor([[[2, 3, 4], [1, 3, 4], [1, 2, 4]],
                                                        [[3, 4, 0], [2, 3, 4], [0, 0, 0]]]))
        target_mask = torch.autograd.Variable(torch.Tensor([[[1, 1, 1], [1, 1, 1], [1, 1, 1]],
                                                            [[1, 1, 0], [1, 1, 1], [0, 0, 0]]]))
        prefix_tree = util.construct_prefix_tree(targets, target_mask)

        # There were two instances in this batch.
        assert len(prefix_tree) == 2

        # The first instance had six valid action sequence prefixes.
        assert len(prefix_tree[0]) == 6
        assert prefix_tree[0][()] == set([1, 2])
        assert prefix_tree[0][(1,)] == set([2, 3])
        assert prefix_tree[0][(1, 2)] == set([4])
        assert prefix_tree[0][(1, 3)] == set([4])
        assert prefix_tree[0][(2,)] == set([3])
        assert prefix_tree[0][(2, 3)] == set([4])

        # The second instance had four valid action sequence prefixes.
        assert len(prefix_tree[1]) == 4
        assert prefix_tree[1][()] == set([2, 3])
        assert prefix_tree[1][(2,)] == set([3])
        assert prefix_tree[1][(2, 3)] == set([4])
        assert prefix_tree[1][(3,)] == set([4])



# pylint: disable=invalid-name,no-self-use,protected-access

from __future__ import absolute_import
from numpy.testing import assert_almost_equal
import numpy
import torch
from torch.autograd import Variable
from torch.nn import Parameter

from allennlp.common import Params
from allennlp.common.testing.test_case import AllenNlpTestCase
from allennlp.modules.attention import LinearAttention
from allennlp.modules.attention.attention import Attention


class LinearAttentionTests(AllenNlpTestCase):

    def test_can_init_linear(self):
        legacy_attention = Attention.from_params(Params({u"type": u"linear",
                                                         u"tensor_1_dim": 3,
                                                         u"tensor_2_dim": 3}))
        isinstance(legacy_attention, LinearAttention)

    def test_linear_similarity(self):
        linear = LinearAttention(3, 3, normalize=True)
        linear._weight_vector = Parameter(torch.FloatTensor([-.3, .5, 2.0, -1.0, 1, 1]))
        linear._bias = Parameter(torch.FloatTensor([.1]))
        output = linear(Variable(torch.FloatTensor([[-7, -8, -9]])),
                        Variable(torch.FloatTensor([[[1, 2, 3], [4, 5, 6]]])))

        assert_almost_equal(output.data.numpy(), numpy.array([[0.0474, 0.9526]]), decimal=2)




from __future__ import absolute_import
#overrides

from allennlp.common.util import JsonDict
from allennlp.data import Instance
from allennlp.predictors.predictor import Predictor

class SimpleSeq2SeqPredictor(Predictor):
    u"""
    Predictor for the :class:`~allennlp.models.encoder_decoder.simple_seq2seq` model.
    """

    def predict(self, source     )            :
        return self.predict_json({u"source" : source})

    #overrides
    def _json_to_instance(self, json_dict          )            :
        u"""
        Expects JSON that looks like ``{"source": "..."}``.
        """
        source = json_dict[u"source"]
        return self._dataset_reader.text_to_instance(source)

SimpleSeq2SeqPredictor = Predictor.register(u'simple_seq2seq')(SimpleSeq2SeqPredictor)



u"""
Defines all the types in the WikitablesQuestions domain.
"""

from __future__ import absolute_import
#typing
#overrides

from nltk.sem.logic import Type, BasicType, ANY_TYPE, ComplexType as NltkComplexType

from allennlp.semparse.type_declarations.type_declaration import ComplexType, HigherOrderType
from allennlp.semparse.type_declarations.type_declaration import PlaceholderType, NamedBasicType
from allennlp.semparse.type_declarations.type_declaration import UnaryOpType, BinaryOpType


class ReverseType(PlaceholderType, HigherOrderType):
    u"""
    ReverseType is a kind of ``PlaceholderType`` where type resolution involves matching the return
    type with the reverse of the argument type. So all we care about are the types of the surrounding
    expressions, and return a resolution that matches whatever parts are present in the type signatures
    of the arguments and the return expressions.

    Following are the resolutions for some example type signatures being matched against::

        <?, <e,r>>      :   <<r,e>, <e,r>>
        <<r,?>, <e,?>>  :   <<r,e>, <e,r>>
        <<r,?>, ?>      :   <<r,?>, <?,r>>>
        <<r,?>, <?,e>>  :   None
    """
    def __init__(self, first      , second      )        :
        super(ReverseType, self).__init__(num_arguments=1, first=first, second=second)
        self._signature = u'<<#1,#2>,<#2,#1>>'

    #overrides
    def resolve(self, other      )                  :
        # Idea: Since its signature is <<#1,#2>,<#2,#1>> no information about types in self is
        # relevant.  All that matters is that other.first resolves against the reverse of
        # other.second and vice versa.
        if not isinstance(other, NltkComplexType):
            return None
        # other.first and other.second are the argument and return types respectively.
        reversed_second = ComplexType(other.second.second, other.second.first)
        other_first = other.first.resolve(reversed_second)
        if not other_first:
            return None
        reversed_first = ComplexType(other_first.second, other_first.first)
        other_second = other.second.resolve(reversed_first)
        if not other_second:
            return None
        return ReverseType(other_first, other_second)

    #overrides
    def get_application_type(self, argument_type      )        :
        return ComplexType(argument_type.second, argument_type.first)

    #overrides
    def substitute_any_type(self, basic_types                )              :
        basic_first_types = basic_types if self.first.first == ANY_TYPE else set([self.first.first])
        basic_second_types = basic_types if self.first.second == ANY_TYPE else set([self.first.second])
        substitutions = []
        for first_type in basic_first_types:
            for second_type in basic_second_types:
                substituted_first = ComplexType(first_type, second_type)
                substituted_second = ComplexType(second_type, first_type)
                substitutions.append(ReverseType(substituted_first, substituted_second))
        return substitutions


class ArgExtremeType(PlaceholderType):
    u"""
    This is the type for argmax and argmin in Sempre. The type signature is <d,<d,<#1,<<d,#1>,#1>>>>.
    Example: (argmax (number 1) (number 1) (fb:row.row.league fb:cell.usl_a_league) fb:row.row.index)
    meaning, of the subset of rows where league == usl_a_league, find the row with the maximum index.
    """
    def __init__(self, basic_type            = ANY_TYPE, lambda_arg_type            = ANY_TYPE)        :
        super(ArgExtremeType, self).__init__(NUMBER_TYPE,
                         ComplexType(NUMBER_TYPE,
                                     ComplexType(basic_type,
                                                 ComplexType(ComplexType(lambda_arg_type, basic_type),
                                                             basic_type))))
        self._signature = u'<n,<n,<#1,<<#2,#1>,#1>>>>'

    #overrides
    def resolve(self, other      )                  :
        u"""See ``PlaceholderType.resolve``"""
        if not isinstance(other, NltkComplexType):
            return None
        expected_second = ComplexType(NUMBER_TYPE,
                                      ComplexType(ANY_TYPE, ComplexType(ComplexType(ANY_TYPE, ANY_TYPE),
                                                                        ANY_TYPE)))
        resolved_second = other.second.resolve(expected_second)
        if resolved_second is None:
            return None

        # The lambda function that we use inside the argmax  must take either a number or a date as
        # an argument.
        lambda_arg_type = other.second.second.second.first.first
        if lambda_arg_type.resolve(NUMBER_TYPE) is None and lambda_arg_type.resolve(DATE_TYPE) is None:
            return None

        try:
            # This is the first #1 in the type signature above.
            selector_function_type = resolved_second.second.first
            # This is the second #1 in the type signature above.
            quant_function_argument_type = resolved_second.second.second.first.second
            # This is the third #1 in the type signature above.
            return_type = resolved_second.second.second.second

            # All three placeholder (ph) types above should resolve against each other.
            resolved_first_ph = selector_function_type.resolve(quant_function_argument_type)
            resolved_first_ph.resolve(return_type)

            resolved_second_ph = quant_function_argument_type.resolve(resolved_first_ph)
            resolved_second_ph.resolve(return_type)

            resolved_third_ph = return_type.resolve(resolved_first_ph)
            resolved_third_ph = return_type.resolve(resolved_second_ph)

            if not resolved_first_ph or not resolved_second_ph or not resolved_third_ph:
                return None

            return ArgExtremeType(resolved_first_ph, lambda_arg_type)
        except AttributeError:
            return None

    #overrides
    def get_application_type(self, argument_type      )        :
        # Returning <d,<#1,<<d,#1>,#1>>>.
        # This is called after the placeholders are resolved.
        return self.second

    #overrides
    def substitute_any_type(self, basic_types                )              :
        if self.second.second.first != ANY_TYPE:
            return [self]
        return [ArgExtremeType(basic_type, inner_function_type)
                for basic_type in basic_types
                for inner_function_type in set([NUMBER_TYPE, DATE_TYPE])]


class CountType(PlaceholderType):
    u"""
    Type of a function that counts arbitrary things. Signature is <#1,n>.
    """
    def __init__(self, count_type      )        :
        super(CountType, self).__init__(count_type, NUMBER_TYPE)
        self._signature = u'<#1,n>'

    #overrides
    def resolve(self, other      )        :
        u"""See ``PlaceholderType.resolve``"""
        if not isinstance(other, NltkComplexType):
            return None
        resolved_second = NUMBER_TYPE.resolve(other.second)
        if not resolved_second:
            return None
        return CountType(other.first)

    #overrides
    def get_application_type(self, argument_type      )        :
        return NUMBER_TYPE

    #overrides
    def substitute_any_type(self, basic_types                )              :
        if self.first != ANY_TYPE:
            return [self]
        return [CountType(basic_type) for basic_type in basic_types]


CELL_TYPE = NamedBasicType(u"CELL")
PART_TYPE = NamedBasicType(u"PART")
ROW_TYPE = NamedBasicType(u"ROW")
DATE_TYPE = NamedBasicType(u"DATE")
NUMBER_TYPE = NamedBasicType(u"NUMBER")

BASIC_TYPES = set([CELL_TYPE, PART_TYPE, ROW_TYPE, DATE_TYPE, NUMBER_TYPE])
# Functions like fb:row.row.year.
COLUMN_TYPE = ComplexType(CELL_TYPE, ROW_TYPE)
# fb:cell.cell.part
PART_TO_CELL_TYPE = ComplexType(PART_TYPE, CELL_TYPE)
# fb:cell.cell.date
DATE_TO_CELL_TYPE = ComplexType(DATE_TYPE, CELL_TYPE)
# fb:cell.cell.number
NUM_TO_CELL_TYPE = ComplexType(NUMBER_TYPE, CELL_TYPE)
# number
NUMBER_FUNCTION_TYPE = ComplexType(NUMBER_TYPE, NUMBER_TYPE)
# date (Signature: <e,<e,<e,d>>>; Example: (date 1982 -1 -1))
DATE_FUNCTION_TYPE = ComplexType(NUMBER_TYPE,
                                 ComplexType(NUMBER_TYPE, ComplexType(NUMBER_TYPE, DATE_TYPE)))
# Unary numerical operations: max, min, >, <, sum etc.
UNARY_DATE_NUM_OP_TYPE = UnaryOpType(allowed_substitutions=set([DATE_TYPE, NUMBER_TYPE]),
                                     signature=u'<nd,nd>')
UNARY_NUM_OP_TYPE = ComplexType(NUMBER_TYPE, NUMBER_TYPE)

# Binary numerical operation: -
BINARY_NUM_OP_TYPE = ComplexType(NUMBER_TYPE, ComplexType(NUMBER_TYPE, NUMBER_TYPE))

# next
ROW_TO_ROW_TYPE = ComplexType(ROW_TYPE, ROW_TYPE)
# reverse
REVERSE_TYPE = ReverseType(ComplexType(ANY_TYPE, ANY_TYPE), ComplexType(ANY_TYPE, ANY_TYPE))
# !=, fb:type.object.type
# fb:type.object.type takes a type and returns all objects of that type.
IDENTITY_TYPE = UnaryOpType()
# index
ROW_INDEX_TYPE = ComplexType(NUMBER_TYPE, ROW_TYPE)
# count
COUNT_TYPE = CountType(ANY_TYPE)
# and, or
CONJUNCTION_TYPE = BinaryOpType()
# argmax, argmin
ARG_EXTREME_TYPE = ArgExtremeType()


COMMON_NAME_MAPPING = {u"lambda": u"\\", u"var": u"V", u"x": u"X"}


COMMON_TYPE_SIGNATURE = {u"V": IDENTITY_TYPE, u"X": ANY_TYPE}


def add_common_name_with_type(name, mapping, type_signature):
    COMMON_NAME_MAPPING[name] = mapping
    COMMON_TYPE_SIGNATURE[mapping] = type_signature

add_common_name_with_type(u"reverse", u"R", REVERSE_TYPE)
add_common_name_with_type(u"argmax", u"A0", ARG_EXTREME_TYPE)
add_common_name_with_type(u"argmin", u"A1", ARG_EXTREME_TYPE)
add_common_name_with_type(u"max", u"M0", UNARY_DATE_NUM_OP_TYPE)
add_common_name_with_type(u"min", u"M1", UNARY_DATE_NUM_OP_TYPE)
add_common_name_with_type(u"and", u"A", CONJUNCTION_TYPE)
add_common_name_with_type(u"or", u"O", CONJUNCTION_TYPE)
add_common_name_with_type(u"fb:row.row.next", u"N", ROW_TO_ROW_TYPE)
add_common_name_with_type(u"number", u"I", NUMBER_FUNCTION_TYPE)
add_common_name_with_type(u"date", u"D0", DATE_FUNCTION_TYPE)
add_common_name_with_type(u"fb:cell.cell.part", u"P", PART_TO_CELL_TYPE)
add_common_name_with_type(u"fb:cell.cell.date", u"D1", DATE_TO_CELL_TYPE)
add_common_name_with_type(u"fb:cell.cell.number", u"I1", NUM_TO_CELL_TYPE)
add_common_name_with_type(u"fb:cell.cell.num2", u"I2", NUM_TO_CELL_TYPE)
add_common_name_with_type(u"fb:row.row.index", u"W", ROW_INDEX_TYPE)
add_common_name_with_type(u"fb:type.row", u"T0", ROW_TYPE)
add_common_name_with_type(u"fb:type.object.type", u"T", ROW_TO_ROW_TYPE)
add_common_name_with_type(u"count", u"C", COUNT_TYPE)
add_common_name_with_type(u"!=", u"Q", IDENTITY_TYPE)
add_common_name_with_type(u">", u"G0", UNARY_DATE_NUM_OP_TYPE)
add_common_name_with_type(u">=", u"G1", UNARY_DATE_NUM_OP_TYPE)
add_common_name_with_type(u"<", u"L0", UNARY_DATE_NUM_OP_TYPE)
add_common_name_with_type(u"<=", u"L1", UNARY_DATE_NUM_OP_TYPE)
add_common_name_with_type(u"sum", u"S0", UNARY_NUM_OP_TYPE)
add_common_name_with_type(u"avg", u"S1", UNARY_NUM_OP_TYPE)
add_common_name_with_type(u"-", u"F", BINARY_NUM_OP_TYPE)  # subtraction




from __future__ import absolute_import
import re
#typing

#overrides
import spacy

from allennlp.common import Registrable
from allennlp.common.util import get_spacy_model
from allennlp.data.tokenizers.token import Token


class WordSplitter(Registrable):
    u"""
    A ``WordSplitter`` splits strings into words.  This is typically called a "tokenizer" in NLP,
    because splitting strings into characters is trivial, but we use ``Tokenizer`` to refer to the
    higher-level object that splits strings into tokens (which could just be character tokens).
    So, we're using "word splitter" here for this.
    """
    default_implementation = u'spacy'

    def batch_split_words(self, sentences           )                     :
        u"""
        Spacy needs to do batch processing, or it can be really slow.  This method lets you take
        advantage of that if you want.  Default implementation is to just iterate of the sentences
        and call ``split_words``, but the ``SpacyWordSplitter`` will actually do batched
        processing.
        """
        return [self.split_words(sentence) for sentence in sentences]

    def split_words(self, sentence     )               :
        u"""
        Splits ``sentence`` into a list of :class:`Token` objects.
        """
        raise NotImplementedError


class SimpleWordSplitter(WordSplitter):
    u"""
    Does really simple tokenization.  NLTK was too slow, so we wrote our own simple tokenizer
    instead.  This just does an initial split(), followed by some heuristic filtering of each
    whitespace-delimited token, separating contractions and punctuation.  We assume lower-cased,
    reasonably well-formed English sentences as input.
    """
    def __init__(self):
        # These are certainly incomplete.  But at least it's a start.
        self.special_cases = set([u'mr.', u'mrs.', u'etc.', u'e.g.', u'cf.', u'c.f.', u'eg.', u'al.'])
        self.contractions = set([u"n't", u"'s", u"'ve", u"'re", u"'ll", u"'d", u"'m"])
        self.contractions |= set([x.replace(u"'", u"'") for x in self.contractions])
        self.ending_punctuation = set([u'"', u"'", u'.', u',', u';', u')', u']', u'}', u':', u'!', u'?', u'%', u'"', u"'"])
        self.beginning_punctuation = set([u'"', u"'", u'(', u'[', u'{', u'#', u'$', u'"', u"'"])

    #overrides
    def split_words(self, sentence     )               :
        u"""
        Splits a sentence into word tokens.  We handle four kinds of things: words with punctuation
        that should be ignored as a special case (Mr. Mrs., etc.), contractions/genitives (isn't,
        don't, Matt's), and beginning and ending punctuation ("antennagate", (parentheticals), and
        such.).

        The basic outline is to split on whitespace, then check each of these cases.  First, we
        strip off beginning punctuation, then strip off ending punctuation, then strip off
        contractions.  When we strip something off the beginning of a word, we can add it to the
        list of tokens immediately.  When we strip it off the end, we have to save it to be added
        to after the word itself has been added.  Before stripping off any part of a token, we
        first check to be sure the token isn't in our list of special cases.
        """
        fields = sentence.split()
        tokens              = []
        for field in fields:
            add_at_end              = []
            while self._can_split(field) and field[0] in self.beginning_punctuation:
                tokens.append(Token(field[0]))
                field = field[1:]
            while self._can_split(field) and field[-1] in self.ending_punctuation:
                add_at_end.insert(0, Token(field[-1]))
                field = field[:-1]

            # There could (rarely) be several contractions in a word, but we check contractions
            # sequentially, in a random order.  If we've removed one, we need to check again to be
            # sure there aren't others.
            remove_contractions = True
            while remove_contractions:
                remove_contractions = False
                for contraction in self.contractions:
                    if self._can_split(field) and field.lower().endswith(contraction):
                        add_at_end.insert(0, Token(field[-len(contraction):]))
                        field = field[:-len(contraction)]
                        remove_contractions = True
            if field:
                tokens.append(Token(field))
            tokens.extend(add_at_end)
        return tokens

    def _can_split(self, token     ):
        return token and token.lower() not in self.special_cases


SimpleWordSplitter = WordSplitter.register(u'simple')(SimpleWordSplitter)

class LettersDigitsWordSplitter(WordSplitter):
    u"""
    A ``WordSplitter`` which keeps runs of (unicode) letters and runs of digits together, while
    every other non-whitespace character becomes a separate word.
    """
    #overrides
    def split_words(self, sentence     )               :
        # We use the [^\W\d_] pattern as a trick to match unicode letters
        tokens = [Token(m.group(), idx=m.start())
                  for m in re.finditer(r'[^\W\d_]+|\d+|\S', sentence)]
        return tokens


LettersDigitsWordSplitter = WordSplitter.register(u'letters_digits')(LettersDigitsWordSplitter)

class JustSpacesWordSplitter(WordSplitter):
    u"""
    A ``WordSplitter`` that assumes you've already done your own tokenization somehow and have
    separated the tokens by spaces.  We just split the input string on whitespace and return the
    resulting list.  We use a somewhat odd name here to avoid coming too close to the more
    commonly used ``SpacyWordSplitter``.

    Note that we use ``sentence.split()``, which means that the amount of whitespace between the
    tokens does not matter.  This will never result in spaces being included as tokens.
    """
    #overrides
    def split_words(self, sentence     )               :
        return [Token(t) for t in sentence.split()]


JustSpacesWordSplitter = WordSplitter.register(u'just_spaces')(JustSpacesWordSplitter)

def _remove_spaces(tokens                          )                            :
    return [token for token in tokens if not token.is_space]

class SpacyWordSplitter(WordSplitter):
    u"""
    A ``WordSplitter`` that uses spaCy's tokenizer.  It's fast and reasonable - this is the
    recommended ``WordSplitter``.
    """
    def __init__(self,
                 language      = u'en_core_web_sm',
                 pos_tags       = False,
                 parse       = False,
                 ner       = False)        :
        self.spacy = get_spacy_model(language, pos_tags, parse, ner)

    #overrides
    def batch_split_words(self, sentences           )                     :
        return [_remove_spaces(tokens)
                for tokens in self.spacy.pipe(sentences, n_threads=-1)]

    #overrides
    def split_words(self, sentence     )               :
        # This works because our Token class matches spacy's.
        return _remove_spaces(self.spacy(sentence))

SpacyWordSplitter = WordSplitter.register(u'spacy')(SpacyWordSplitter)




from __future__ import absolute_import
from collections import defaultdict
#typing

#overrides

import torch
from torch.nn import Parameter
from torch.nn.modules.rnn import LSTMCell
from torch.nn.modules.linear import Linear

from allennlp.common import util as common_util
from allennlp.common.checks import check_dimensions_match
from allennlp.models.semantic_parsing.wikitables.wikitables_decoder_state import WikiTablesDecoderState
from allennlp.modules import Attention, FeedForward
from allennlp.modules.token_embedders import Embedding
from allennlp.nn import util
from allennlp.nn.decoding import DecoderStep, RnnState
try:
    from itertools import izip
except:
    izip = zip



class WikiTablesDecoderStep(DecoderStep[WikiTablesDecoderState]):
    u"""
    Parameters
    ----------
    encoder_output_dim : ``int``
    action_embedding_dim : ``int``
    input_attention : ``Attention``
    num_start_types : ``int``
    num_entity_types : ``int``
    mixture_feedforward : ``FeedForward`` (optional, default=None)
    dropout : ``float`` (optional, default=0.0)
    unlinked_terminal_indices : ``List[int]``, (optional, default=None)
        If we are training a parser to maximize coverage using a checklist, we need to know the
        global indices of the unlinked terminal productions to be able to compute the checklist
        corresponding to those terminals, and project a concatenation of the current hidden
        state, attended encoder input and the current checklist balance into the action space.
        This is not needed if we are training the parser using target action sequences.
    """
    def __init__(self,
                 encoder_output_dim     ,
                 action_embedding_dim     ,
                 input_attention           ,
                 num_start_types     ,
                 num_entity_types     ,
                 mixture_feedforward              = None,
                 dropout        = 0.0,
                 unlinked_terminal_indices            = None)        :
        super(WikiTablesDecoderStep, self).__init__()
        self._mixture_feedforward = mixture_feedforward
        self._entity_type_embedding = Embedding(num_entity_types, action_embedding_dim)
        self._input_attention = input_attention

        self._num_start_types = num_start_types
        self._start_type_predictor = Linear(encoder_output_dim, num_start_types)

        # Decoder output dim needs to be the same as the encoder output dim since we initialize the
        # hidden state of the decoder with the final hidden state of the encoder.
        output_dim = encoder_output_dim
        input_dim = output_dim
        # Our decoder input will be the concatenation of the decoder hidden state and the previous
        # action embedding, and we'll project that down to the decoder's `input_dim`, which we
        # arbitrarily set to be the same as `output_dim`.
        self._input_projection_layer = Linear(output_dim + action_embedding_dim, input_dim)
        # Before making a prediction, we'll compute an attention over the input given our updated
        # hidden state. Then we concatenate those with the decoder state and project to
        # `action_embedding_dim` to make a prediction.
        self._output_projection_layer = Linear(output_dim + encoder_output_dim, action_embedding_dim)
        if unlinked_terminal_indices is not None:
            # This means we are using coverage to train the parser.
            # These factors are used to add the embeddings of yet to be produced actions to the
            # predicted embedding, and to boost the action logits of yet to be produced linked
            # actions, respectively.
            self._unlinked_checklist_multiplier = Parameter(torch.FloatTensor([1.0]))
            self._linked_checklist_multiplier = Parameter(torch.FloatTensor([1.0]))

        self._unlinked_terminal_indices = unlinked_terminal_indices
        # TODO(pradeep): Do not hardcode decoder cell type.
        self._decoder_cell = LSTMCell(input_dim, output_dim)

        if mixture_feedforward is not None:
            check_dimensions_match(output_dim, mixture_feedforward.get_input_dim(),
                                   u"hidden state embedding dim", u"mixture feedforward input dim")
            check_dimensions_match(mixture_feedforward.get_output_dim(), 1,
                                   u"mixture feedforward output dim", u"dimension for scalar value")

        if dropout > 0:
            self._dropout = torch.nn.Dropout(p=dropout)
        else:
            self._dropout = lambda x: x

    #overrides
    def take_step(self,
                  state                        ,
                  max_actions      = None,
                  allowed_actions                 = None)                                :
        if not state.action_history[0]:
            # The wikitables parser did something different when predicting the start type, which
            # is our first action.  So in this case we break out into a different function.  We'll
            # ignore max_actions on our first step, assuming there aren't that many start types.
            return self._take_first_step(state, allowed_actions)
        # Outline here: first we'll construct the input to the decoder, which is a concatenation of
        # an embedding of the decoder input (the last action taken) and an attention over the
        # question.  Then we'll update our decoder's hidden state given this input, and recompute an
        # attention over the question given our new hidden state.  We'll use a concatenation of the
        # new hidden state and the new attention, and optionally the checklist balance, to predict an
        # output, then yield new states.  Each new state corresponds to one valid action that can be
        # taken from the current state, and they are ordered by their probability of being selected.
        attended_question = torch.stack([rnn_state.attended_input for rnn_state in state.rnn_state])
        hidden_state = torch.stack([rnn_state.hidden_state for rnn_state in state.rnn_state])
        memory_cell = torch.stack([rnn_state.memory_cell for rnn_state in state.rnn_state])
        previous_action_embedding = torch.stack([rnn_state.previous_action_embedding
                                                 for rnn_state in state.rnn_state])

        # The scores from all prior state transitions until now.  Shape: (group_size, 1).
        scores_so_far = torch.stack(state.score).unsqueeze(-1)

        # (group_size, decoder_input_dim)
        projected_input = self._input_projection_layer(torch.cat([attended_question,
                                                                  previous_action_embedding], -1))
        decoder_input = torch.nn.functional.relu(projected_input)

        hidden_state, memory_cell = self._decoder_cell(decoder_input, (hidden_state, memory_cell))
        hidden_state = self._dropout(hidden_state)

        # (group_size, encoder_output_dim)
        encoder_outputs = torch.stack([state.rnn_state[0].encoder_outputs[i] for i in state.batch_indices])
        encoder_output_mask = torch.stack([state.rnn_state[0].encoder_output_mask[i] for i in state.batch_indices])
        attended_question, attention_weights = self.attend_on_question(hidden_state,
                                                                       encoder_outputs,
                                                                       encoder_output_mask)
        action_query = torch.cat([hidden_state, attended_question], dim=-1)
        considered_actions, actions_to_embed, actions_to_link = self._get_actions_to_consider(state)
        # action_embeddings: (group_size, num_embedded_actions, action_embedding_dim)
        # output_action_embeddings: (group_size, num_embedded_actions, action_embedding_dim)
        # action_mask: (group_size, num_embedded_actions)
        action_embeddings, output_action_embeddings, action_biases, embedded_action_mask =\
                self._get_action_embeddings(state, actions_to_embed)
        # (group_size, action_embedding_dim)
        projected_query = torch.nn.functional.relu(self._output_projection_layer(action_query))
        predicted_action_embedding = self._dropout(projected_query)
        linked_balance = None
        if state.checklist_state[0] is not None:
            linked_balance, unlinked_balance = self._get_checklist_balance(state,
                                                                           self._unlinked_terminal_indices,
                                                                           actions_to_link)
            embedding_addition = self._get_predicted_embedding_addition(state,
                                                                        self._unlinked_terminal_indices,
                                                                        unlinked_balance)
            addition = embedding_addition * self._unlinked_checklist_multiplier
            predicted_action_embedding = predicted_action_embedding + addition

        # We'll do a batch dot product here with `bmm`.  We want `dot(predicted_action_embedding,
        # action_embedding)` for each `action_embedding`, and we can get that efficiently with
        # `bmm` and some squeezing.
        # Shape: (group_size, num_embedded_actions)
        embedded_action_logits = action_embeddings.bmm(predicted_action_embedding.unsqueeze(-1)).squeeze(-1)
        embedded_action_logits = embedded_action_logits + action_biases.squeeze(-1)
        if actions_to_link:
            # entity_action_logits: (group_size, num_entity_actions)
            # entity_action_mask: (group_size, num_entity_actions)
            entity_action_logits, entity_action_mask, entity_type_embeddings =\
                    self._get_entity_action_logits(state,
                                                   actions_to_link,
                                                   attention_weights,
                                                   linked_balance)

            # The `output_action_embeddings` tensor gets used later as the input to the next
            # decoder step.  For linked actions, we don't have any action embedding, so we use the
            # entity type instead.
            output_action_embeddings = torch.cat([output_action_embeddings, entity_type_embeddings], dim=1)

            if self._mixture_feedforward is not None:
                # The entity and action logits are combined with a mixture weight to prevent the
                # entity_action_logits from dominating the embedded_action_logits if a softmax
                # was applied on both together.
                mixture_weight = self._mixture_feedforward(hidden_state)
                mix1 = torch.log(mixture_weight)
                mix2 = torch.log(1 - mixture_weight)

                entity_action_probs = util.masked_log_softmax(entity_action_logits,
                                                              entity_action_mask.float()) + mix1
                embedded_action_probs = util.masked_log_softmax(embedded_action_logits,
                                                                embedded_action_mask.float()) + mix2
                current_log_probs = torch.cat([embedded_action_probs, entity_action_probs], dim=1)
            else:
                action_logits = torch.cat([embedded_action_logits, entity_action_logits], dim=1)
                action_mask = torch.cat([embedded_action_mask, entity_action_mask], dim=1).float()
                current_log_probs = util.masked_log_softmax(action_logits, action_mask)
        else:
            action_logits = embedded_action_logits
            action_mask = embedded_action_mask.float()
            current_log_probs = util.masked_log_softmax(action_logits, action_mask)

        # current_log_probs is shape (group_size, num_actions).  We're broadcasting an addition
        # here with scores_so_far, which has shape (group_size, 1).  This is now the total score
        # for each state after taking each action.  We're going to sort by this in
        # `_compute_new_states`, so it's important that this is the total score, not just the score
        # for the current action.
        log_probs = scores_so_far + current_log_probs

        return self._compute_new_states(state,
                                        log_probs,
                                        hidden_state,
                                        memory_cell,
                                        output_action_embeddings,
                                        attended_question,
                                        attention_weights,
                                        considered_actions,
                                        allowed_actions,
                                        max_actions)

    @staticmethod
    def _get_checklist_balance(state                        ,
                               unlinked_terminal_indices           ,
                               actions_to_link                 ):                            
                                                                                            
        # This holds a list of checklist balances for this state. Each balance is a float vector
        # containing just 1s and 0s showing which of the items are filled. We clamp the min at 0
        # to ignore the number of times an action is taken. The value at an index will be 1 iff
        # the target wants an unmasked action to be taken, and it is not yet taken. All elements
        # in each balance corresponding to masked actions will be 0.
        checklist_balances = []
        for instance_checklist_state in state.checklist_state:
            checklist_balance = torch.clamp(instance_checklist_state.get_balance(), min=0.0)
            checklist_balances.append(checklist_balance)

        checklist_balance = torch.stack([x for x in  checklist_balances])
        checklist_balance = checklist_balance.squeeze(2)  # (group_size, num_terminals)
        # We now need to split the ``checklist_balance`` into two tensors, one corresponding to
        # linked actions and the other to unlinked actions because they affect the output action
        # logits differently. We use ``unlinked_terminal_indices`` and ``actions_to_link`` to do that, but
        # the indices in those lists are indices of all actions, and the checklist balance
        # corresponds only to the terminal actions.
        # To make things more confusing, ``actions_to_link`` has batch action indices, and
        # ``unlinked_terminal_indices`` has global action indices.
        mapped_actions_to_link = []
        mapped_actions_to_embed = []
        # Mapping from batch action indices to checklist indices for each instance in group.
        batch_actions_to_checklist = [checklist_state.terminal_indices_dict
                                      for checklist_state in state.checklist_state]
        for group_index, batch_index in enumerate(state.batch_indices):
            instance_mapped_embedded_actions = []
            for action in unlinked_terminal_indices:
                batch_action_index = state.global_to_batch_action_indices[(batch_index, action)]
                if batch_action_index in batch_actions_to_checklist[group_index]:
                    checklist_index = batch_actions_to_checklist[group_index][batch_action_index]
                else:
                    # This means that the embedded action is not a terminal, because the checklist
                    # indices only correspond to terminal actions.
                    checklist_index = -1
                instance_mapped_embedded_actions.append(checklist_index)
            mapped_actions_to_embed.append(instance_mapped_embedded_actions)
        # We don't need to pad the unlinked actions because they're all currently the
        # same size as ``unlinked_terminal_indices``.
        unlinked_action_indices = checklist_balance.new_tensor(mapped_actions_to_embed, dtype=torch.long)
        unlinked_actions_mask = (unlinked_action_indices != -1).long()
        # torch.gather would complain if the indices are -1. So making them all 0 now. We'll use the
        # mask again on the balances.
        unlinked_action_indices = unlinked_action_indices * unlinked_actions_mask

        unlinked_checklist_balance = torch.gather(checklist_balance, 1, unlinked_action_indices)
        unlinked_checklist_balance = unlinked_checklist_balance * unlinked_actions_mask.float()
        # If actions_to_link is None, it means that all the valid actions in the current state need
        # to be embedded. We simply return None for checklist balance corresponding to linked
        # actions then.
        linked_checklist_balance = None
        if actions_to_link:
            for group_index, instance_actions_to_link in enumerate(actions_to_link):
                mapped_actions_to_link.append([batch_actions_to_checklist[group_index][action]
                                               for action in instance_actions_to_link])
            # We need to pad the linked action indices before we use them to gather appropriate balances.
            # Some of the indices may be 0s. So we need to make the padding index -1.
            max_num_linked_actions = max([len(indices) for indices in mapped_actions_to_link])
            padded_actions_to_link = [common_util.pad_sequence_to_length(indices,
                                                                         max_num_linked_actions,
                                                                         default_value=lambda: -1)
                                      for indices in mapped_actions_to_link]
            linked_action_indices = checklist_balance.new_tensor(padded_actions_to_link, dtype=torch.long)
            linked_actions_mask = (linked_action_indices != -1).long()
            linked_action_indices = linked_action_indices * linked_actions_mask
            linked_checklist_balance = torch.gather(checklist_balance, 1, linked_action_indices)
            linked_checklist_balance = linked_checklist_balance * linked_actions_mask.float()
        return linked_checklist_balance, unlinked_checklist_balance

    @staticmethod
    def _get_predicted_embedding_addition(state                        ,
                                          unlinked_terminal_indices           ,
                                          unlinked_checklist_balance              )                :
        u"""
        Gets the embeddings of desired unlinked terminal actions yet to be produced by the decoder,
        and returns their sum for the decoder to add it to the predicted embedding to bias the
        prediction towards missing actions.
        """
        # (group_size, num_unlinked_actions, 1)
        unlinked_balance = unlinked_checklist_balance.unsqueeze(2)
        group_size = len(state.batch_indices)
        action_embedding_dim = state.action_embeddings.size(-1)
        num_terminals = len(unlinked_terminal_indices)
        group_terminal_indices = [unlinked_terminal_indices for _ in range(group_size)]
        # (group_size, num_unlinked_actions)
        terminal_indices_tensor = state.score[0].new_tensor(group_terminal_indices, dtype=torch.long)
        flattened_terminal_indices = terminal_indices_tensor.view(-1)
        flattened_action_embeddings = state.action_embeddings.index_select(0,
                                                                           flattened_terminal_indices)
        terminal_embeddings = flattened_action_embeddings.view(group_size, num_terminals, action_embedding_dim)
        checklist_balance_embeddings = terminal_embeddings * unlinked_balance
        # (group_size, action_embedding_dim)
        return checklist_balance_embeddings.sum(1)

    def _take_first_step(self,
                         state                        ,
                         allowed_actions                 = None)                                :
        # We'll just do a projection from the current hidden state (which was initialized with the
        # final encoder output) to the number of start actions that we have, normalize those
        # logits, and use that as our score.  We end up duplicating some of the logic from
        # `_compute_new_states` here, but we do things slightly differently, and it's easier to
        # just copy the parts we need than to try to re-use that code.

        # (group_size, hidden_dim)
        hidden_state = torch.stack([rnn_state.hidden_state for rnn_state in state.rnn_state])
        # (group_size, num_start_type)
        start_action_logits = self._start_type_predictor(hidden_state)
        log_probs = util.masked_log_softmax(start_action_logits, None)
        sorted_log_probs, sorted_actions = log_probs.sort(dim=-1, descending=True)

        sorted_actions = sorted_actions.detach().cpu().numpy().tolist()
        if state.debug_info is not None:
            probs_cpu = log_probs.exp().detach().cpu().numpy().tolist()

        # state.get_valid_actions() will return a list that is consistently sorted, so as along as
        # the set of valid start actions never changes, we can just match up the log prob indices
        # above with the position of each considered action, and we're good.
        considered_actions, _, _ = self._get_actions_to_consider(state)
        if len(considered_actions[0]) != self._num_start_types:
            raise RuntimeError(u"Calculated wrong number of initial actions.  Expected "
                               "{self._num_start_types}, found {len(considered_actions[0])}.")

        best_next_states                                        = defaultdict(list)
        for group_index, (batch_index, group_actions) in enumerate(izip(state.batch_indices, sorted_actions)):
            for action_index, action in enumerate(group_actions):
                # `action` is currently the index in `log_probs`, not the actual action ID.  To get
                # the action ID, we need to go through `considered_actions`.
                action = considered_actions[group_index][action]
                if allowed_actions is not None and action not in allowed_actions[group_index]:
                    # This happens when our _decoder trainer_ wants us to only evaluate certain
                    # actions, likely because they are the gold actions in this state.  We just skip
                    # emitting any state that isn't allowed by the trainer, because constructing the
                    # new state can be expensive.
                    continue
                best_next_states[batch_index].append((group_index, action_index, action))

        new_states = []
        for batch_index, best_states in sorted(best_next_states.items()):
            for group_index, action_index, action in best_states:
                # We'll yield a bunch of states here that all have a `group_size` of 1, so that the
                # learning algorithm can decide how many of these it wants to keep, and it can just
                # regroup them later, as that's a really easy operation.
                batch_index = state.batch_indices[group_index]
                new_action_history = state.action_history[group_index] + [action]
                new_score = state.score[group_index] + sorted_log_probs[group_index, action_index]

                production_rule = state.possible_actions[batch_index][action][0]
                new_grammar_state = state.grammar_state[group_index].take_action(production_rule)
                new_checklist_state = [state.checklist_state[group_index]]
                if state.debug_info is not None:
                    debug_info = {
                            u'considered_actions': considered_actions[group_index],
                            u'probabilities': probs_cpu[group_index],
                            }
                    new_debug_info = [state.debug_info[group_index] + [debug_info]]
                else:
                    new_debug_info = None

                # This part is different from `_compute_new_states` - we're just passing through
                # the previous RNN state, as predicting the start type wasn't included in the
                # decoder RNN in the original model.
                new_rnn_state = state.rnn_state[group_index]
                new_state = WikiTablesDecoderState(batch_indices=[batch_index],
                                                   action_history=[new_action_history],
                                                   score=[new_score],
                                                   rnn_state=[new_rnn_state],
                                                   grammar_state=[new_grammar_state],
                                                   action_embeddings=state.action_embeddings,
                                                   output_action_embeddings=state.output_action_embeddings,
                                                   action_biases=state.action_biases,
                                                   action_indices=state.action_indices,
                                                   possible_actions=state.possible_actions,
                                                   flattened_linking_scores=state.flattened_linking_scores,
                                                   actions_to_entities=state.actions_to_entities,
                                                   entity_types=state.entity_types,
                                                   world=state.world,
                                                   example_lisp_string=state.example_lisp_string,
                                                   checklist_state=new_checklist_state,
                                                   debug_info=new_debug_info)
                new_states.append(new_state)
        return new_states

    def attend_on_question(self,
                           query              ,
                           encoder_outputs              ,
                           encoder_output_mask              )                                     :
        u"""
        Given a query (which is typically the decoder hidden state), compute an attention over the
        output of the question encoder, and return a weighted sum of the question representations
        given this attention.  We also return the attention weights themselves.

        This is a simple computation, but we have it as a separate method so that the ``forward``
        method on the main parser module can call it on the initial hidden state, to simplify the
        logic in ``take_step``.
        """
        # (group_size, question_length)
        question_attention_weights = self._input_attention(query,
                                                           encoder_outputs,
                                                           encoder_output_mask)
        # (group_size, encoder_output_dim)
        attended_question = util.weighted_sum(encoder_outputs, question_attention_weights)
        return attended_question, question_attention_weights

    @staticmethod
    def _get_actions_to_consider(state                        ):                          
                                                                                         
                                                                                         
        u"""
        The ``WikiTablesDecoderState`` defines a set of actions that are valid in the current
        grammar state for each group element.  This method gets that set of actions and separates
        them into actions that can be embedded and actions that need to be linked.

        This method goes through all of the actions from ``state.get_valid_actions()`` and
        separates them into actions that can be embedded and actions that need to be linked, based
        on the action's ``global_action_index`` (all embeddable actions have an action index lower
        than the number of global embeddable actions).  After separating the actions, we combine
        them again, getting a padded list of all considered actions that can be used by
        :func:`_compute_new_states`.  All three of these lists are returned (the embeddable
        actions, the actions that need to be linked, and the padded collection of all actions that
        were considered).

        Returns
        -------
        considered_actions : ``List[List[int]]``
            A sorted list of all actions considered for each group element, both for embedding and
            for linking.  This list has one inner list for each group element, and each item in the
            inner list represents ``batch_action_index`` that was considered.  This inner list is
            also `padded` to size ``max_num_embedded_actions + max_num_linked_actions``, with
            `interior` padding in between the embedded actions and the linked actions where
            necessary.  The ``action_index`` for padded entries is -1.  This padding replicates the
            structre that we'll get in the model once we concatenate logits together, so that
            :func:`_compute_new_states` has an easy time figuring out what to do.
        actions_to_embed : ``List[List[int]]``
            These actions are in the global action embedding tensor, and can be embedded.  Shape is
            (group_size, num_actions), not padded, and the value is the ``global_action_index``,
            not the ``batch_action_index``.  You can use these indices to ``index_select`` on the
            global action embeddings directly, without additional translation.
        actions_to_link : ``List[List[int]]``
            These actions are `not` in the global action embedding tensor, and must have scores
            computed some way other than with an embedding.  Shape is (group_size, num_actions),
            not padded, and the value is the ``batch_action_index``.  These need to be converted
            into batch entity indices, then looked up in the linking scores.

            If there are `no` actions to link, because all actions have an embedding, we return
            `None` here.
        """
        # A list of `batch_action_indices` for each group element.
        valid_actions = state.get_valid_actions()
        global_valid_actions                              = []
        for batch_index, valid_action_list in izip(state.batch_indices, valid_actions):
            global_valid_actions.append([])
            for action_index in valid_action_list:
                # state.action_indices is a dictionary that maps (batch_index, batch_action_index)
                # to global_action_index
                global_action_index = state.action_indices[(batch_index, action_index)]
                global_valid_actions[-1].append((global_action_index, action_index))
        embedded_actions                  = []
        linked_actions                  = []
        for global_action_list in global_valid_actions:
            embedded_actions.append([])
            linked_actions.append([])
            for global_action_index, action_index in global_action_list:
                if global_action_index == -1:
                    linked_actions[-1].append(action_index)
                else:
                    embedded_actions[-1].append(global_action_index)
        num_embedded_actions = max(len(actions) for actions in embedded_actions)
        num_linked_actions = max(len(actions) for actions in linked_actions)
        if num_linked_actions == 0:
            linked_actions = None
        considered_actions                  = []
        for global_action_list in global_valid_actions:
            considered_actions.append([])
            # First we add the embedded actions to the list.
            for global_action_index, action_index in global_action_list:
                if global_action_index != -1:
                    considered_actions[-1].append(action_index)
            # Then we pad that portion.
            while len(considered_actions[-1]) < num_embedded_actions:
                considered_actions[-1].append(-1)
            # Then we add the linked actions to the list.
            for global_action_index, action_index in global_action_list:
                if global_action_index == -1:
                    considered_actions[-1].append(action_index)
            # Finally, we pad the linked portion.
            while len(considered_actions[-1]) < num_embedded_actions + num_linked_actions:
                considered_actions[-1].append(-1)
        return considered_actions, embedded_actions, linked_actions

    @staticmethod
    def _get_action_embeddings(state                        ,
                               actions_to_embed                 ):                       
                                                                                        
                                                                                        
                                                                                        
        u"""
        Returns an embedded representation for all actions in ``actions_to_embed``, using the state
        in ``WikiTablesDecoderState``.

        Parameters
        ----------
        state : ``WikiTablesDecoderState``
            The current state.  We'll use this to get the global action embeddings.
        actions_to_embed : ``List[List[int]]``
            A list of _global_ action indices for each group element.  Should have shape
            (group_size, num_actions), unpadded.  This is expected to be output from
            :func:`_get_actions_to_consider`.

        Returns
        -------
        action_embeddings : ``torch.FloatTensor``
            An embedded representation of all of the given actions.  Shape is ``(group_size,
            num_actions, action_embedding_dim)``, where ``num_actions`` is the maximum number of
            considered actions for any group element.
        output_action_embeddings : ``torch.FloatTensor``
            A second embedded representation of all of the given actions.  The first is used when
            selecting actions, the second is used as the decoder output (which is the input at the
            next timestep).  This is similar to having separate word embeddings and softmax layer
            weights in a language model or MT model.
        action_biases : ``torch.FloatTensor``
            A bias weight for predicting each action.  Shape is ``(group_size, num_actions, 1)``.
        action_mask : ``torch.LongTensor``
            A mask of shape ``(group_size, num_actions)`` indicating which ``(group_index,
            action_index)`` pairs were merely added as padding.
        """
        num_actions = [len(action_list) for action_list in actions_to_embed]
        max_num_actions = max(num_actions)
        padded_actions = [common_util.pad_sequence_to_length(action_list, max_num_actions)
                          for action_list in actions_to_embed]
        # Shape: (group_size, num_actions)
        action_tensor = state.score[0].new_tensor(padded_actions, dtype=torch.long)
        # `state.action_embeddings` is shape (total_num_actions, action_embedding_dim).
        # We want to select from state.action_embeddings using `action_tensor` to get a tensor of
        # shape (group_size, num_actions, action_embedding_dim).  Unfortunately, the index_select
        # functions in nn.util don't do this operation.  So we'll do some reshapes and do the
        # index_select ourselves.
        group_size = len(state.batch_indices)
        action_embedding_dim = state.action_embeddings.size(-1)

        flattened_actions = action_tensor.view(-1)
        flattened_action_embeddings = state.action_embeddings.index_select(0, flattened_actions)
        action_embeddings = flattened_action_embeddings.view(group_size, max_num_actions, action_embedding_dim)

        flattened_output_embeddings = state.output_action_embeddings.index_select(0, flattened_actions)
        output_embeddings = flattened_output_embeddings.view(group_size, max_num_actions, action_embedding_dim)

        flattened_biases = state.action_biases.index_select(0, flattened_actions)
        biases = flattened_biases.view(group_size, max_num_actions, 1)

        sequence_lengths = action_embeddings.new_tensor(num_actions)
        action_mask = util.get_mask_from_sequence_lengths(sequence_lengths, max_num_actions)
        return action_embeddings, output_embeddings, biases, action_mask

    def _get_entity_action_logits(self,
                                  state                        ,
                                  actions_to_link                 ,
                                  attention_weights              ,
                                  linked_checklist_balance               = None):                            
                                                                                                           
                                                                                                            
        u"""
        Returns scores for each action in ``actions_to_link`` that are derived from the linking
        scores between the question and the table entities, and the current attention on the
        question.  The intuition is that if we're paying attention to a particular word in the
        question, we should tend to select entity productions that we think that word refers to.
        We additionally return a mask representing which elements in the returned ``action_logits``
        tensor are just padding, and an embedded representation of each action that can be used as
        input to the next step of the encoder.  That embedded representation is derived from the
        type of the entity produced by the action.

        The ``actions_to_link`` are in terms of the `batch` action list passed to
        ``model.forward()``.  We need to convert these integers into indices into the linking score
        tensor, which has shape (batch_size, num_entities, num_question_tokens), look up the
        linking score for each entity, then aggregate the scores using the current question
        attention.

        Parameters
        ----------
        state : ``WikiTablesDecoderState``
            The current state.  We'll use this to get the linking scores.
        actions_to_link : ``List[List[int]]``
            A list of _batch_ action indices for each group element.  Should have shape
            (group_size, num_actions), unpadded.  This is expected to be output from
            :func:`_get_actions_to_consider`.
        attention_weights : ``torch.Tensor``
            The current attention weights over the question tokens.  Should have shape
            ``(group_size, num_question_tokens)``.
        linked_checklist_balance : ``torch.Tensor``, optional (default=None)
            If the parser is being trained to maximize coverage over an agenda, this is the balance
            vector corresponding to entity actions, containing 1s and 0s, with 1s showing the
            actions that are yet to be produced. Required only if the parser is being trained to
            maximize coverage.

        Returns
        -------
        action_logits : ``torch.FloatTensor``
            A score for each of the given actions.  Shape is ``(group_size, num_actions)``, where
            ``num_actions`` is the maximum number of considered actions for any group element.
        action_mask : ``torch.LongTensor``
            A mask of shape ``(group_size, num_actions)`` indicating which ``(group_index,
            action_index)`` pairs were merely added as padding.
        type_embeddings : ``torch.LongTensor``
            A tensor of shape ``(group_size, num_actions, action_embedding_dim)``, with an embedded
            representation of the `type` of the entity corresponding to each action.
        """
        # First we map the actions to entity indices, using state.actions_to_entities, and find the
        # type of each entity using state.entity_types.
        action_entities                  = []
        entity_types                  = []
        for batch_index, action_list in izip(state.batch_indices, actions_to_link):
            action_entities.append([])
            entity_types.append([])
            for action_index in action_list:
                entity_index = state.actions_to_entities[(batch_index, action_index)]
                action_entities[-1].append(entity_index)
                entity_types[-1].append(state.entity_types[entity_index])

        # Then we create a padded tensor suitable for use with
        # `state.flattened_linking_scores.index_select()`.
        num_actions = [len(action_list) for action_list in action_entities]
        max_num_actions = max(num_actions)
        padded_actions = [common_util.pad_sequence_to_length(action_list, max_num_actions)
                          for action_list in action_entities]
        padded_types = [common_util.pad_sequence_to_length(type_list, max_num_actions)
                        for type_list in entity_types]
        # Shape: (group_size, num_actions)
        action_tensor = state.score[0].new_tensor(padded_actions, dtype=torch.long)
        type_tensor = state.score[0].new_tensor(padded_types, dtype=torch.long)

        # To get the type embedding tensor, we just use an embedding matrix on the list of entity
        # types.
        type_embeddings = self._entity_type_embedding(type_tensor)
        # `state.flattened_linking_scores` is shape (batch_size * num_entities, num_question_tokens).
        # We want to select from this using `action_tensor` to get a tensor of shape (group_size,
        # num_actions, num_question_tokens).  Unfortunately, the index_select functions in nn.util
        # don't do this operation.  So we'll do some reshapes and do the index_select ourselves.
        group_size = len(state.batch_indices)
        num_question_tokens = state.flattened_linking_scores.size(-1)
        flattened_actions = action_tensor.view(-1)
        # (group_size * num_actions, num_question_tokens)
        flattened_action_linking = state.flattened_linking_scores.index_select(0, flattened_actions)
        # (group_size, num_actions, num_question_tokens)
        action_linking = flattened_action_linking.view(group_size, max_num_actions, num_question_tokens)

        # Now we get action logits by weighting these entity x token scores by the attention over
        # the question tokens.  We can do this efficiently with torch.bmm.
        action_logits = action_linking.bmm(attention_weights.unsqueeze(-1)).squeeze(-1)
        if linked_checklist_balance is not None:
            # ``linked_checklist_balance`` is a binary tensor of size (group_size, num_actions) with
            # 1s indicating the linked actions that the agenda wants the decoder to produce, but
            # haven't been produced yet. We're simply doubling the logits of those actions here.
            action_logits_addition = action_logits * linked_checklist_balance
            action_logits = action_logits + self._linked_checklist_multiplier * action_logits_addition
        # Finally, we make a mask for our action logit tensor.
        sequence_lengths = action_linking.new_tensor(num_actions)
        action_mask = util.get_mask_from_sequence_lengths(sequence_lengths, max_num_actions)
        return action_logits, action_mask, type_embeddings

    @staticmethod
    def _compute_new_states(state                        ,
                            log_probs              ,
                            hidden_state              ,
                            memory_cell              ,
                            action_embeddings              ,
                            attended_question              ,
                            attention_weights              ,
                            considered_actions                 ,
                            allowed_actions                ,
                            max_actions      = None)                                :
        # Each group index here might get accessed multiple times, and doing the slicing operation
        # each time is more expensive than doing it once upfront.  These three lines give about a
        # 10% speedup in training time.  I also tried this with sorted_log_probs and
        # action_embeddings, but those get accessed for _each action_, so doing the splits there
        # didn't help.
        hidden_state = [x.squeeze(0) for x in hidden_state.split(1, 0)]
        memory_cell = [x.squeeze(0) for x in memory_cell.split(1, 0)]
        attended_question = [x.squeeze(0) for x in attended_question.split(1, 0)]

        sorted_log_probs, sorted_actions = log_probs.sort(dim=-1, descending=True)
        if max_actions is not None:
            # We might need a version of `sorted_log_probs` on the CPU later, but only if we need
            # to truncate the best states to `max_actions`.
            sorted_log_probs_cpu = sorted_log_probs.detach().cpu().numpy()
        if state.debug_info is not None:
            probs_cpu = log_probs.exp().detach().cpu().numpy().tolist()
        sorted_actions = sorted_actions.detach().cpu().numpy().tolist()
        best_next_states                                        = defaultdict(list)
        for group_index, (batch_index, group_actions) in enumerate(izip(state.batch_indices,
                                                                       sorted_actions)):
            for action_index, action in enumerate(group_actions):
                # `action` is currently the index in `log_probs`, not the actual action ID.  To get
                # the action ID, we need to go through `considered_actions`.
                action = considered_actions[group_index][action]
                if action == -1:
                    # This was padding.
                    continue
                if allowed_actions is not None and action not in allowed_actions[group_index]:
                    # This happens when our _decoder trainer_ wants us to only evaluate certain
                    # actions, likely because they are the gold actions in this state.  We just skip
                    # emitting any state that isn't allowed by the trainer, because constructing the
                    # new state can be expensive.
                    continue
                best_next_states[batch_index].append((group_index, action_index, action))
        new_states = []
        for batch_index, best_states in sorted(best_next_states.items()):
            if max_actions is not None:
                # We sorted previously by _group_index_, but we then combined by _batch_index_.  We
                # need to get the top next states for each _batch_ instance, so we sort all of the
                # instance's states again (across group index) by score.  We don't need to do this
                # if `max_actions` is None, because we'll be keeping all of the next states,
                # anyway.
                best_states.sort(key=lambda x: sorted_log_probs_cpu[x[:2]], reverse=True)
                best_states = best_states[:max_actions]
            for group_index, action_index, action in best_states:
                # We'll yield a bunch of states here that all have a `group_size` of 1, so that the
                # learning algorithm can decide how many of these it wants to keep, and it can just
                # regroup them later, as that's a really easy operation.
                batch_index = state.batch_indices[group_index]
                new_action_history = state.action_history[group_index] + [action]
                new_score = sorted_log_probs[group_index, action_index]

                # `action_index` is the index in the _sorted_ tensors, but the action embedding
                # matrix is _not_ sorted, so we need to get back the original, non-sorted action
                # index before we get the action embedding.
                action_embedding_index = sorted_actions[group_index][action_index]
                action_embedding = action_embeddings[group_index, action_embedding_index, :]
                production_rule = state.possible_actions[batch_index][action][0]
                new_grammar_state = state.grammar_state[group_index].take_action(production_rule)
                if state.checklist_state[0] is not None:
                    new_checklist_state = [state.checklist_state[group_index].update(action)]
                else:
                    new_checklist_state = None
                if state.debug_info is not None:
                    debug_info = {
                            u'considered_actions': considered_actions[group_index],
                            u'question_attention': attention_weights[group_index],
                            u'probabilities': probs_cpu[group_index],
                            }
                    new_debug_info = [state.debug_info[group_index] + [debug_info]]
                else:
                    new_debug_info = None

                new_rnn_state = RnnState(hidden_state[group_index],
                                         memory_cell[group_index],
                                         action_embedding,
                                         attended_question[group_index],
                                         state.rnn_state[group_index].encoder_outputs,
                                         state.rnn_state[group_index].encoder_output_mask)
                new_state = WikiTablesDecoderState(batch_indices=[batch_index],
                                                   action_history=[new_action_history],
                                                   score=[new_score],
                                                   rnn_state=[new_rnn_state],
                                                   grammar_state=[new_grammar_state],
                                                   action_embeddings=state.action_embeddings,
                                                   output_action_embeddings=state.output_action_embeddings,
                                                   action_biases=state.action_biases,
                                                   action_indices=state.action_indices,
                                                   possible_actions=state.possible_actions,
                                                   flattened_linking_scores=state.flattened_linking_scores,
                                                   actions_to_entities=state.actions_to_entities,
                                                   entity_types=state.entity_types,
                                                   world=state.world,
                                                   example_lisp_string=state.example_lisp_string,
                                                   checklist_state=new_checklist_state,
                                                   debug_info=new_debug_info)
                new_states.append(new_state)
        return new_states



# pytest: disable=no-self-use,invalid-name



from __future__ import division
from __future__ import with_statement
from __future__ import absolute_import
import shutil
import sys

import pytest

from allennlp.commands import main
from allennlp.commands.subcommand import Subcommand
from allennlp.common.checks import ConfigurationError
from allennlp.common.testing import AllenNlpTestCase
from io import open

class TestMain(AllenNlpTestCase):
    def test_fails_on_unknown_command(self):
        sys.argv = [u"bogus",         # command
                    u"unknown_model", # model_name
                    u"bogus file",    # input_file
                    u"--output-file", u"bogus out file",
                    u"--silent"]

        with self.assertRaises(SystemExit) as cm:  # pylint: disable=invalid-name
            main()

        assert cm.exception.code == 2  # argparse code for incorrect usage

    def test_subcommand_overrides(self):
        def do_nothing(_):
            pass

        class FakeEvaluate(Subcommand):
            add_subparser_called = False

            def add_subparser(self, name, parser):
                subparser = parser.add_parser(name,
                                              description=u"fake",
                                              help=u"fake help")

                subparser.set_defaults(func=do_nothing)
                self.add_subparser_called = True

                return subparser

        fake_evaluate = FakeEvaluate()

        sys.argv = [u"allennlp.run", u"evaluate"]
        main(subcommand_overrides={u"evaluate": fake_evaluate})

        assert fake_evaluate.add_subparser_called

    def test_other_modules(self):
        # Create a new package in a temporary dir
        packagedir = self.TEST_DIR / u'testpackage'
        packagedir.mkdir()  # pylint: disable=no-member
        (packagedir / u'__init__.py').touch()  # pylint: disable=no-member

        # And add that directory to the path
        sys.path.insert(0, unicode(self.TEST_DIR))

        # Write out a duplicate model there, but registered under a different name.
        from allennlp.models import simple_tagger
        with open(simple_tagger.__file__) as model_file:
            code = model_file.read().replace(u"""@Model.register("simple_tagger")""",
                                             u"""@Model.register("duplicate-test-tagger")""")

        with open(packagedir / u'model.py', u'w') as new_model_file:
            new_model_file.write(code)

        # Copy fixture there too.
        shutil.copy(self.FIXTURES_ROOT / u'data' / u'sequence_tagging.tsv', self.TEST_DIR)
        data_path = unicode(self.TEST_DIR / u'sequence_tagging.tsv')

        # Write out config file
        config_path = self.TEST_DIR / u'config.json'
        config_json = u"""{
                "model": {
                        "type": "duplicate-test-tagger",
                        "text_field_embedder": {
                                "tokens": {
                                        "type": "embedding",
                                        "embedding_dim": 5
                                }
                        },
                        "encoder": {
                                "type": "lstm",
                                "input_size": 5,
                                "hidden_size": 7,
                                "num_layers": 2
                        }
                },
                "dataset_reader": {"type": "sequence_tagging"},
                "train_data_path": "$$$",
                "validation_data_path": "$$$",
                "iterator": {"type": "basic", "batch_size": 2},
                "trainer": {
                        "num_epochs": 2,
                        "optimizer": "adam"
                }
            }""".replace(u'$$$', data_path)
        with open(config_path, u'w') as config_file:
            config_file.write(config_json)

        serialization_dir = self.TEST_DIR / u'serialization'

        # Run train with using the non-allennlp module.
        sys.argv = [u"bin/allennlp",
                    u"train", unicode(config_path),
                    u"-s", unicode(serialization_dir)]

        # Shouldn't be able to find the model.
        with pytest.raises(ConfigurationError):
            main()

        # Now add the --include-package flag and it should work.
        # We also need to add --recover since the output directory already exists.
        sys.argv.extend([u"--recover", u"--include-package", u'testpackage'])

        main()

        # Rewrite out config file, but change a value.
        with open(config_path, u'w') as new_config_file:
            new_config_file.write(config_json.replace(u'"num_epochs": 2,', u'"num_epochs": 4,'))

        # This should fail because the config.json does not match that in the serialization directory.
        with pytest.raises(ConfigurationError):
            main()

        sys.path.remove(unicode(self.TEST_DIR))
